@@ -12,7 +12,7 @@ static GList *rate_pads = NULL;
 static GList *seekable_elements = NULL;
 
 static GstElement *pipeline;
-static guint64 duration;
+static gint64 duration;
 static GtkAdjustment *adjustment;
 static GtkWidget *hscale;
 static gboolean stats = FALSE;
@@ -663,7 +663,7 @@ query_rates (void)
 }
 
 G_GNUC_UNUSED static void
-query_durations_elems ()
+query_durations_elems (void)
 {
   GList *walk = seekable_elements;
 
@@ -693,7 +693,7 @@ query_durations_elems ()
 }
 
 G_GNUC_UNUSED static void
-query_durations_pads ()
+query_durations_pads (void)
 {
   GList *walk = seekable_pads;
 
@@ -723,7 +723,7 @@ query_durations_pads ()
 }
 
 G_GNUC_UNUSED static void
-query_positions_elems ()
+query_positions_elems (void)
 {
   GList *walk = seekable_elements;
 
@@ -753,7 +753,7 @@ query_positions_elems ()
 }
 
 G_GNUC_UNUSED static void
-query_positions_pads ()
+query_positions_pads (void)
 {
   GList *walk = seekable_pads;
 
@@ -786,7 +786,7 @@ static gboolean
 update_scale (gpointer data)
 {
   GstClock *clock;
-  guint64 position;
+  gint64 position;
   GstFormat format = GST_FORMAT_TIME;
   gboolean res;
 
@@ -1035,11 +1035,11 @@ main (int argc, char **argv)
   gtk_scale_set_digits (GTK_SCALE (hscale), 2);
   gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_CONTINUOUS);
 
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (hscale,
       "button_press_event", G_CALLBACK (start_seek), pipeline);
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (hscale,
       "button_release_event", G_CALLBACK (stop_seek), pipeline);
-  gtk_signal_connect (GTK_OBJECT (hscale),
+  g_signal_connect (hscale,
       "format_value", G_CALLBACK (format_value), pipeline);
 
   /* do the packing stuff ... */
