@@ -200,7 +200,7 @@ enum
 static void gst_gnomevfssrc_base_init (gpointer g_class);
 static void gst_gnomevfssrc_class_init (GstGnomeVFSSrcClass * klass);
 static void gst_gnomevfssrc_init (GstGnomeVFSSrc * gnomevfssrc);
-static void gst_gnomevfssrc_dispose (GObject * object);
+static void gst_gnomevfssrc_finalize (GObject * object);
 
 static void gst_gnomevfssrc_uri_handler_init (gpointer g_iface,
     gpointer iface_data);
@@ -284,7 +284,7 @@ gst_gnomevfssrc_class_init (GstGnomeVFSSrcClass * klass)
       "bytesperread", ARG_BYTESPERREAD, G_PARAM_READWRITE,
       "location", ARG_LOCATION, G_PARAM_READWRITE, NULL);
 
-  gobject_class->dispose = gst_gnomevfssrc_dispose;
+  gobject_class->finalize = gst_gnomevfssrc_finalize;
 
   g_object_class_install_property (gobject_class,
       ARG_HANDLE,
@@ -381,7 +381,7 @@ gst_gnomevfssrc_init (GstGnomeVFSSrc * gnomevfssrc)
 }
 
 static void
-gst_gnomevfssrc_dispose (GObject * object)
+gst_gnomevfssrc_finalize (GObject * object)
 {
   GstGnomeVFSSrc *src = GST_GNOMEVFSSRC (object);
 
@@ -407,7 +407,7 @@ gst_gnomevfssrc_dispose (GObject * object)
   g_mutex_free (src->audiocast_udpdata_mutex);
   g_mutex_free (src->audiocast_queue_mutex);
 
-  G_OBJECT_CLASS (parent_class)->dispose (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static guint
@@ -624,7 +624,7 @@ audiocast_init (GstGnomeVFSSrc * src)
 
   if (!src->iradio_mode)
     return TRUE;
-  GST_DEBUG ("audiocast: registering listener");
+  GST_DEBUG_OBJECT (src, "audiocast: registering listener");
   if (audiocast_register_listener (&src->audiocast_port,
           &src->audiocast_fd) < 0) {
     GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
@@ -632,7 +632,7 @@ audiocast_init (GstGnomeVFSSrc * src)
     close (src->audiocast_fd);
     return FALSE;
   }
-  GST_DEBUG ("audiocast: creating pipe");
+  GST_DEBUG_OBJECT (src, "audiocast: creating pipe");
   src->audiocast_notify_queue = NULL;
   if (pipe (pipefds) < 0) {
     close (src->audiocast_fd);
@@ -640,7 +640,7 @@ audiocast_init (GstGnomeVFSSrc * src)
   }
   src->audiocast_thread_die_infd = pipefds[0];
   src->audiocast_thread_die_outfd = pipefds[1];
-  GST_DEBUG ("audiocast: creating audiocast thread");
+  GST_DEBUG_OBJECT (src, "audiocast: creating audiocast thread");
   src->audiocast_thread =
       g_thread_create ((GThreadFunc) audiocast_thread_run, src, TRUE, &error);
   if (error != NULL) {
@@ -719,22 +719,26 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
   FD_SET (src->audiocast_thread_die_infd, &fdset);
 
   while (1) {
-    GST_DEBUG ("audiocast thread: dropping into select");
+    GST_DEBUG_OBJECT (src, "audiocast thread: dropping into select");
     readset = fdset;
     if (select (FD_SETSIZE, &readset, NULL, NULL, NULL) < 0) {
-      perror ("select");
+      GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+        ("Unable to select"));
       return NULL;
     }
     if (FD_ISSET (src->audiocast_thread_die_infd, &readset)) {
       char buf[1];
 
-      GST_DEBUG ("audiocast thread: got die character");
-      read (src->audiocast_thread_die_infd, buf, 1);
+      GST_DEBUG_OBJECT (src, "audiocast thread: got die character");
+      if (read (src->audiocast_thread_die_infd, buf, 1) != 1) {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+          ("Unable to read die character"));
+      }
       close (src->audiocast_thread_die_infd);
       close (src->audiocast_fd);
       return NULL;
     }
-    GST_DEBUG ("audiocast thread: reading data");
+    GST_DEBUG_OBJECT (src, "audiocast thread: reading data");
     len =
         recvfrom (src->audiocast_fd, buf, sizeof (buf) - 1, 0,
         (struct sockaddr *) &from, &fromlen);
@@ -780,7 +784,7 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
           g_mutex_lock (src->audiocast_queue_mutex);
           src->audiocast_notify_queue =
               g_list_append (src->audiocast_notify_queue, "iradio-title");
-          GST_DEBUG ("audiocast title: %s\n", src->iradio_title);
+          GST_DEBUG_OBJECT (src, "audiocast title: %s\n", src->iradio_title);
           g_mutex_unlock (src->audiocast_queue_mutex);
         } else if (!strncmp (lines[i], "x-audiocast-streamurl", 21)) {
           g_mutex_lock (src->audiocast_udpdata_mutex);
@@ -791,7 +795,7 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
           g_mutex_lock (src->audiocast_queue_mutex);
           src->audiocast_notify_queue =
               g_list_append (src->audiocast_notify_queue, "iradio-url");
-          GST_DEBUG ("audiocast url: %s\n", src->iradio_title);
+          GST_DEBUG_OBJECT (src, "audiocast url: %s\n", src->iradio_title);
           g_mutex_unlock (src->audiocast_queue_mutex);
         } else if (!strncmp (lines[i], "x-audiocast-udpseqnr", 20)) {
           gchar outbuf[120];
@@ -805,7 +809,7 @@ audiocast_thread_run (GstGnomeVFSSrc * src)
                 strerror (errno));
             continue;
           }
-          GST_DEBUG ("sent audiocast ack: %s\n", outbuf);
+          GST_DEBUG_OBJECT (src, "sent audiocast ack: %s\n", outbuf);
         }
       }
       g_strfreev (lines);
@@ -826,10 +830,13 @@ audiocast_thread_kill (GstGnomeVFSSrc * src)
      metadata, then we don't need the
      audiocast metadata too.
    */
-  GST_DEBUG ("audiocast: writing die character");
-  write (src->audiocast_thread_die_outfd, "q", 1);
+  GST_DEBUG_OBJECT (src, "audiocast: writing die character");
+  if (write (src->audiocast_thread_die_outfd, "q", 1) != 1) {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_WRITE, (NULL),
+          ("Unable to write die character"));
+  }
   close (src->audiocast_thread_die_outfd);
-  GST_DEBUG ("audiocast: joining thread");
+  GST_DEBUG_OBJECT (src, "audiocast: joining thread");
   g_thread_join (src->audiocast_thread);
   src->audiocast_thread = NULL;
 }
@@ -844,7 +851,7 @@ gst_gnomevfssrc_send_additional_headers_callback (gconstpointer in,
 
   if (!src->iradio_mode)
     return;
-  GST_DEBUG ("sending headers\n");
+  GST_DEBUG_OBJECT (src, "sending headers\n");
 
   out_args->headers = g_list_append (out_args->headers,
       g_strdup ("icy-metadata:1\r\n"));
@@ -881,9 +888,12 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 
     /* Icecast stuff */
     if (!strncmp (data, "icy-metaint:", 12)) {  /* ugh */
-      sscanf (data + 12, "%d", &icy_metaint);
+      if (sscanf (data + 12, "%d", &icy_metaint) != 1) {
+        GST_ELEMENT_ERROR (src, RESOURCE, OPEN_READ, (NULL),
+          ("Could not parse icy-metaint header"));
+      }
       src->icy_metaint = icy_metaint;
-      GST_DEBUG ("got icy-metaint %d, killing audiocast thread",
+      GST_DEBUG_OBJECT (src, "got icy-metaint %d, killing audiocast thread",
           src->icy_metaint);
       audiocast_thread_kill (src);
       continue;
@@ -896,7 +906,7 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
     else
       continue;
 
-    GST_DEBUG ("key: %s", key);
+    GST_DEBUG_OBJECT (src, "key: %s", key);
     if (!strncmp (key, "name", 4)) {
       g_free (src->iradio_name);
       src->iradio_name = gst_gnomevfssrc_unicodify (value);
@@ -917,29 +927,29 @@ gst_gnomevfssrc_received_headers_callback (gconstpointer in,
 }
 
 static void
-gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc * gnomevfssrc)
+gst_gnomevfssrc_push_callbacks (GstGnomeVFSSrc * src)
 {
-  if (gnomevfssrc->http_callbacks_pushed)
+  if (src->http_callbacks_pushed)
     return;
 
-  GST_DEBUG ("pushing callbacks");
+  GST_DEBUG_OBJECT (src, "pushing callbacks");
   gnome_vfs_module_callback_push
       (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS,
-      gst_gnomevfssrc_send_additional_headers_callback, gnomevfssrc, NULL);
+      gst_gnomevfssrc_send_additional_headers_callback, src, NULL);
   gnome_vfs_module_callback_push
       (GNOME_VFS_MODULE_CALLBACK_HTTP_RECEIVED_HEADERS,
-      gst_gnomevfssrc_received_headers_callback, gnomevfssrc, NULL);
+      gst_gnomevfssrc_received_headers_callback, src, NULL);
 
-  gnomevfssrc->http_callbacks_pushed = TRUE;
+  src->http_callbacks_pushed = TRUE;
 }
 
 static void
-gst_gnomevfssrc_pop_callbacks (GstGnomeVFSSrc * gnomevfssrc)
+gst_gnomevfssrc_pop_callbacks (GstGnomeVFSSrc * src)
 {
-  if (!gnomevfssrc->http_callbacks_pushed)
+  if (!src->http_callbacks_pushed)
     return;
 
-  GST_DEBUG ("popping callbacks");
+  GST_DEBUG_OBJECT (src, "popping callbacks");
   gnome_vfs_module_callback_pop
       (GNOME_VFS_MODULE_CALLBACK_HTTP_SEND_ADDITIONAL_HEADERS);
   gnome_vfs_module_callback_pop
@@ -958,7 +968,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   gchar **tags;
   int i;
 
-  GST_DEBUG ("reading icecast metadata");
+  GST_DEBUG_OBJECT (src, "reading icecast metadata");
 
   while (length == 0) {
     res = gnome_vfs_read (src->handle, &foobyte, 1, &length);
@@ -971,7 +981,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   if (metadata_length == 0)
     return;
 
-  data = g_new (gchar, metadata_length + 1);
+  data = g_new (guchar, metadata_length + 1);
   pos = data;
 
   while (pos - data < metadata_length) {
@@ -987,14 +997,14 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
   }
 
   data[metadata_length] = 0;
-  tags = g_strsplit (data, "';", 0);
+  tags = g_strsplit ((gchar *) data, "';", 0);
 
   for (i = 0; tags[i]; i++) {
     if (!g_ascii_strncasecmp (tags[i], "StreamTitle=", 12)) {
       g_free (src->iradio_title);
       src->iradio_title = gst_gnomevfssrc_unicodify (tags[i] + 13);
       if (src->iradio_title) {
-        GST_DEBUG ("sending notification on icecast title");
+        GST_DEBUG_OBJECT (src, "sending notification on icecast title");
         g_object_notify (G_OBJECT (src), "iradio-title");
       } else
         g_print ("Unable to convert icecast title \"%s\" to UTF-8!\n",
@@ -1005,7 +1015,7 @@ gst_gnomevfssrc_get_icy_metadata (GstGnomeVFSSrc * src)
       g_free (src->iradio_url);
       src->iradio_url = gst_gnomevfssrc_unicodify (tags[i] + 11);
       if (src->iradio_url) {
-        GST_DEBUG ("sending notification on icecast url");
+        GST_DEBUG_OBJECT (src, "sending notification on icecast url");
         g_object_notify (G_OBJECT (src), "iradio-url");
       } else
         g_print ("Unable to convert icecast url \"%s\" to UTF-8!\n",
@@ -1040,6 +1050,7 @@ gst_gnomevfssrc_get (GstPad * pad)
   /* deal with EOF state */
   if ((src->curoffset >= src->size) && (src->size != 0)) {
     gst_element_set_eos (GST_ELEMENT (src));
+    GST_DEBUG_OBJECT (src, "Returning EOS");
     return GST_DATA (gst_event_new (GST_EVENT_EOS));
   }
 
@@ -1056,13 +1067,14 @@ gst_gnomevfssrc_get (GstPad * pad)
     g_return_val_if_fail (GST_BUFFER_DATA (buf) != NULL, NULL);
 
     GST_BUFFER_SIZE (buf) = 0;
-    GST_DEBUG ("doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
+    GST_DEBUG_OBJECT (src, "doing read: icy_count: %" G_GINT64_FORMAT, src->icy_count);
     result = gnome_vfs_read (src->handle, data,
         src->icy_metaint - src->icy_count, &readbytes);
 
     /* EOS? */
     if (readbytes == 0) {
       gst_buffer_unref (buf);
+      GST_DEBUG_OBJECT (src, "Returning EOS");
       gst_element_set_eos (GST_ELEMENT (src));
       return GST_DATA (gst_event_new (GST_EVENT_EOS));
     }
@@ -1087,7 +1099,7 @@ gst_gnomevfssrc_get (GstPad * pad)
 
       src->need_flush = FALSE;
       gst_buffer_unref (buf);
-      GST_DEBUG ("gnomevfssrc sending flush");
+      GST_DEBUG_OBJECT (src, "gnomevfssrc sending flush");
       return GST_DATA (event);
     }
 
@@ -1095,9 +1107,9 @@ gst_gnomevfssrc_get (GstPad * pad)
       GstEvent *event;
 
       gst_buffer_unref (buf);
-      GST_DEBUG ("new seek %" G_GINT64_FORMAT, src->curoffset);
+      GST_DEBUG_OBJECT (src, "new seek %" G_GINT64_FORMAT, src->curoffset);
       src->new_seek = FALSE;
-      GST_DEBUG ("gnomevfssrc sending discont");
+      GST_DEBUG_OBJECT (src, "gnomevfssrc sending discont");
       event =
           gst_event_new_discontinuous (FALSE, GST_FORMAT_BYTES, src->curoffset,
           NULL);
@@ -1107,14 +1119,13 @@ gst_gnomevfssrc_get (GstPad * pad)
     result = gnome_vfs_read (src->handle, GST_BUFFER_DATA (buf),
         src->bytes_per_read, &readbytes);
 
-    GST_DEBUG ("read: %s, readbytes: %" G_GINT64_FORMAT,
-        gnome_vfs_result_to_string (result), readbytes);
+    GST_DEBUG_OBJECT (src, "read: %s, readbytes: %" G_GINT64_FORMAT " @ %" G_GINT64_FORMAT,
+        gnome_vfs_result_to_string (result), readbytes, src->curoffset);
     /* deal with EOS */
     if (readbytes == 0) {
       gst_buffer_unref (buf);
-
+      GST_DEBUG_OBJECT (src, "Returning EOS");
       gst_element_set_eos (GST_ELEMENT (src));
-
       return GST_DATA (gst_event_new (GST_EVENT_EOS));
     }
 
@@ -1144,7 +1155,17 @@ gst_gnomevfssrc_open_file (GstGnomeVFSSrc * src)
   gst_gnomevfssrc_push_callbacks (src);
 
   if (src->uri != NULL) {
-    result = gnome_vfs_open_uri (&(src->handle), src->uri, GNOME_VFS_OPEN_READ);
+    result = gnome_vfs_open_uri (&(src->handle), src->uri,
+        GNOME_VFS_OPEN_READ | GNOME_VFS_OPEN_RANDOM);
+
+    if (result == GNOME_VFS_ERROR_INVALID_OPEN_MODE ||
+        result == GNOME_VFS_ERROR_NOT_SUPPORTED) {
+
+      /* seeking might not be supported, try without it */
+      result = gnome_vfs_open_uri (&(src->handle), src->uri,
+          GNOME_VFS_OPEN_READ);
+    }
+
     if (result != GNOME_VFS_OK) {
       gchar *filename = gnome_vfs_uri_to_string (src->uri,
           GNOME_VFS_URI_HIDE_PASSWORD);
@@ -1174,11 +1195,11 @@ gst_gnomevfssrc_open_file (GstGnomeVFSSrc * src)
     if (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_SIZE)
       src->size = info->size;
   } else
-    GST_DEBUG ("getting info failed: %s", gnome_vfs_result_to_string (result));
+    GST_DEBUG_OBJECT (src, "getting info failed: %s", gnome_vfs_result_to_string (result));
 
   gnome_vfs_file_info_unref (info);
 
-  GST_DEBUG ("size %" G_GINT64_FORMAT, src->size);
+  GST_DEBUG_OBJECT (src, "size %" G_GINT64_FORMAT, src->size);
 
   audiocast_do_notifications (src);
 
@@ -1249,7 +1270,7 @@ gst_gnomevfssrc_srcpad_query (GstPad * pad, GstQueryType type,
 
   switch (type) {
     case GST_QUERY_TOTAL:
-      if (*format != GST_FORMAT_BYTES) {
+      if (*format != GST_FORMAT_BYTES || src->size == 0) {
         return FALSE;
       }
       *value = src->size;
@@ -1298,6 +1319,8 @@ gst_gnomevfssrc_srcpad_event (GstPad * pad, GstEvent * event)
           desired_offset = src->curoffset + GST_EVENT_SEEK_OFFSET (event);
           break;
         case GST_SEEK_METHOD_END:
+          if (src->size == 0)
+            return FALSE;
           desired_offset = src->size - ABS (GST_EVENT_SEEK_OFFSET (event));
           break;
         default:
@@ -1308,7 +1331,8 @@ gst_gnomevfssrc_srcpad_event (GstPad * pad, GstEvent * event)
 
       result = gnome_vfs_seek (src->handle,
           GNOME_VFS_SEEK_START, desired_offset);
-      GST_DEBUG ("new_seek: %s", gnome_vfs_result_to_string (result));
+      GST_DEBUG_OBJECT (src, "new_seek to %" G_GINT64_FORMAT ": %s",
+          desired_offset, gnome_vfs_result_to_string (result));
 
       if (result != GNOME_VFS_OK) {
         gst_event_unref (event);
