@@ -83,6 +83,7 @@ struct _GstMad
   gint index_id;
 
   gboolean check_for_xing;
+  gboolean xing_found;
 };
 
 struct _GstMadClass
@@ -365,6 +366,7 @@ gst_mad_init (GstMad * mad)
   mad->half = FALSE;
   mad->ignore_crc = TRUE;
   mad->check_for_xing = TRUE;
+  mad->xing_found = FALSE;
   GST_FLAG_SET (mad, GST_ELEMENT_EVENT_AWARE);
 }
 
@@ -752,10 +754,17 @@ gst_mad_src_event (GstPad * pad, GstEvent * event)
       break;
       /* the all-formats seek logic */
     case GST_EVENT_SEEK:
-      if (mad->index)
-        res = index_seek (mad, pad, event);
-      else
-        res = normal_seek (mad, pad, event);
+      GST_DEBUG ("forwarding seek event to sink pad");
+      gst_event_ref (event);
+      if (gst_pad_send_event (GST_PAD_PEER (mad->sinkpad), event)) {
+        /* seek worked, we're done, loop will exit */
+        res = TRUE;
+      } else {
+        if (mad->index)
+          res = index_seek (mad, pad, event);
+        else
+          res = normal_seek (mad, pad, event);
+      }
       break;
 
     default:
@@ -840,7 +849,7 @@ gst_mad_update_info (GstMad * mad)
 G_STMT_START{							\
   if (mad->header.h1 != header->h1 || mad->new_header) {	\
     mad->header.h1 = header->h1;				\
-     changed = TRUE;						\
+    changed = TRUE;						\
   };								\
 } G_STMT_END
 
@@ -858,10 +867,7 @@ G_STMT_START{							\
   CHECK_HEADER (layer, "layer");
   CHECK_HEADER (mode, "mode");
   CHECK_HEADER (emphasis, "emphasis");
-
-  if (header->bitrate != mad->header.bitrate || mad->new_header) {
-    mad->header.bitrate = header->bitrate;
-  }
+  mad->header.bitrate = header->bitrate;
   mad->new_header = FALSE;
 
   if (changed) {
@@ -880,11 +886,13 @@ G_STMT_START{							\
         GST_TAG_LAYER, mad->header.layer,
         GST_TAG_MODE, mode->value_nick,
         GST_TAG_EMPHASIS, emphasis->value_nick, NULL);
-    gst_element_found_tags (GST_ELEMENT (mad), list);
-    gst_tag_list_free (list);
+    if (!mad->xing_found) {
+      gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+          GST_TAG_BITRATE, mad->header.bitrate, NULL);
+    }
+    gst_element_found_tags_for_pad (GST_ELEMENT (mad), mad->srcpad, 0, list);
   }
 #undef CHECK_HEADER
-
 }
 
 static void
@@ -910,7 +918,7 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
         if (gst_formats_contains (formats, GST_EVENT_DISCONT_OFFSET (event,
                     i).format)) {
           gint64 value = GST_EVENT_DISCONT_OFFSET (event, i).value;
-          gint64 time;
+          gint64 time, total_samples;
           GstFormat format;
           GstEvent *discont;
 
@@ -932,9 +940,10 @@ gst_mad_handle_event (GstPad * pad, GstBuffer * buffer)
            * that this doesn't happen anywhere so far). */
           format = GST_FORMAT_DEFAULT;
           if (!gst_pad_convert (mad->srcpad,
-                  GST_FORMAT_TIME, time, &format, &mad->total_samples)) {
+                  GST_FORMAT_TIME, time, &format, &total_samples)) {
             continue;
           }
+          mad->total_samples = total_samples;
 
           if (GST_PAD_IS_USABLE (mad->srcpad)) {
             discont = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME,
@@ -1126,11 +1135,11 @@ mpg123_parse_xing_header (struct mad_header *header,
 
 /* internal function to check if the header has changed and thus the
  * caps need to be reset.  Only call during normal mode, not resyncing */
-static void
+static gboolean
 gst_mad_check_caps_reset (GstMad * mad)
 {
   guint nchannels;
-  guint rate;
+  guint rate, old_rate = mad->rate;
 
   nchannels = MAD_NCHANNELS (&mad->frame.header);
 
@@ -1139,6 +1148,12 @@ gst_mad_check_caps_reset (GstMad * mad)
 #else
   rate = mad->frame.header.samplerate;
 #endif
+  if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE) {
+    GST_INFO_OBJECT (mad,
+        "MAD_OPTION_HALFSAMPLERATE is set, adapting rate from %u to %u", rate,
+        rate >> 1);
+    rate >>= 1;
+  }
 
   /* rate and channels are not supposed to change in a continuous stream,
    * so check this first before doing anything */
@@ -1157,8 +1172,10 @@ gst_mad_check_caps_reset (GstMad * mad)
         mad->pending_channels = nchannels;
         mad->pending_rate = rate;
       }
+      /* Now, we already have a valid caps set and will continue to use
+       * that for a while longer, so we cans afely return TRUE here. */
       if (++mad->times_pending < 3)
-        return;
+        return TRUE;
     }
   }
   gst_mad_update_info (mad);
@@ -1166,9 +1183,6 @@ gst_mad_check_caps_reset (GstMad * mad)
   if (mad->channels != nchannels || mad->rate != rate) {
     GstCaps *caps;
 
-    if (mad->stream.options & MAD_OPTION_HALFSAMPLERATE)
-      rate >>= 1;
-
     /* we set the caps even when the pad is not connected so they
      * can be gotten for streaminfo */
     caps = gst_caps_new_simple ("audio/x-raw-int",
@@ -1182,9 +1196,20 @@ gst_mad_check_caps_reset (GstMad * mad)
       mad->caps_set = TRUE;     /* set back to FALSE on discont */
       mad->channels = nchannels;
       mad->rate = rate;
+
+      /* update sample count so we don't come up with crazy timestamps */
+      if (mad->total_samples && old_rate) {
+        mad->total_samples = mad->total_samples * rate / old_rate;
+      }
+    } else {
+      GST_ELEMENT_ERROR (mad, CORE, NEGOTIATION, (NULL),
+          ("Failed to negotiate %d Hz, %d channels", rate, nchannels));
+      return FALSE;
     }
     gst_caps_free (caps);
   }
+
+  return TRUE;
 }
 
 static void
@@ -1232,7 +1257,7 @@ gst_mad_chain (GstPad * pad, GstData * _data)
   }
 
   /* handle data */
-  data = GST_BUFFER_DATA (buffer);
+  data = (gchar *) GST_BUFFER_DATA (buffer);
   size = GST_BUFFER_SIZE (buffer);
 
   /* process the incoming buffer in chunks of maximum MAD_BUFFER_MDLEN bytes;
@@ -1269,7 +1294,7 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       guint nsamples;
       guint64 time_offset;
       guint64 time_duration;
-      unsigned char const *before_sync, *after_sync;
+      gboolean resync = TRUE;
 
       mad->in_error = FALSE;
 
@@ -1277,12 +1302,6 @@ gst_mad_chain (GstPad * pad, GstData * _data)
 
       /* added separate header decoding to catch errors earlier, also fixes
        * some weird decoding errors... */
-      GST_LOG ("decoding the header now");
-      if (mad_header_decode (&mad->frame.header, &mad->stream) == -1) {
-        GST_DEBUG ("mad_frame_decode had an error: %s",
-            mad_stream_errorstr (&mad->stream));
-      }
-
       GST_LOG ("decoding one frame now");
 
       if (mad_frame_decode (&mad->frame, &mad->stream) == -1) {
@@ -1329,6 +1348,9 @@ gst_mad_chain (GstPad * pad, GstData * _data)
              * id3 tags, so we need to flush one byte less than the tagsize */
             mad_stream_skip (&mad->stream, tagsize - 1);
 
+            /* When we skip, we don't want to call sync */
+            resync = FALSE;
+
             tag = id3_tag_parse (data, tagsize);
             if (tag) {
               GstTagList *list;
@@ -1350,44 +1372,51 @@ gst_mad_chain (GstPad * pad, GstData * _data)
             }
           }
         }
+        //Should not sync here if mad_skip has been used before, the offset
+        //is "pending" inside mad and will be applied on next call to decode.
+        if (resync) {
+          unsigned char const *before_sync, *after_sync;
 
-        mad_frame_mute (&mad->frame);
-        mad_synth_mute (&mad->synth);
-        before_sync = mad->stream.ptr.byte;
-        if (mad_stream_sync (&mad->stream) != 0)
-          GST_WARNING ("mad_stream_sync failed");
-        after_sync = mad->stream.ptr.byte;
-        /* a succesful resync should make us drop bytes as consumed, so
-           calculate from the byte pointers before and after resync */
-        consumed = after_sync - before_sync;
-        GST_DEBUG ("resynchronization consumes %d bytes", consumed);
-        GST_DEBUG ("synced to data: 0x%0x 0x%0x", *mad->stream.ptr.byte,
-            *(mad->stream.ptr.byte + 1));
+          before_sync = mad->stream.ptr.byte;
+          if (mad_stream_sync (&mad->stream) != 0) {
+            consumed = MAD_BUFFER_GUARD < mad->tempsize ?
+                mad->tempsize - MAD_BUFFER_GUARD : 0;
+            GST_DEBUG_OBJECT (mad,
+                "mad_stream_sync failed, skipping all %u bytes we have",
+                consumed);
+          } else {
+            after_sync = mad->stream.ptr.byte;
+            /* a succesful resync should make us drop bytes as consumed, so
+               calculate from the byte pointers before and after resync */
+            consumed = after_sync - before_sync;
+            GST_DEBUG_OBJECT (mad, "resynchronization consumes %d bytes",
+                consumed);
+            GST_DEBUG_OBJECT (mad, "synced to data: 0x%0x 0x%0x",
+                *mad->stream.ptr.byte, *(mad->stream.ptr.byte + 1));
 
+            /* recoverable errors pass */
+          }
+          resync = FALSE;
+        }
 
-        mad_stream_sync (&mad->stream);
-        /* recoverable errors pass */
         goto next;
       }
 
       if (mad->check_for_xing) {
-        int bitrate, time;
+        int bitrate = 0, time = 0;
         GstTagList *list;
         int frame_len = mad->stream.next_frame - mad->stream.this_frame;
 
         /* Assume Xing headers can only be the first frame in a mp3 file */
         if (mpg123_parse_xing_header (&mad->frame.header,
                 mad->stream.this_frame, frame_len, &bitrate, &time)) {
+          mad->xing_found = TRUE;
           list = gst_tag_list_new ();
           gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
               GST_TAG_DURATION, (gint64) time * 1000 * 1000 * 1000,
               GST_TAG_BITRATE, bitrate, NULL);
-          gst_element_found_tags (GST_ELEMENT (mad), list);
-          if (GST_PAD_IS_USABLE (mad->srcpad)) {
-            gst_pad_push (mad->srcpad, GST_DATA (gst_event_new_tag (list)));
-          } else {
-            gst_tag_list_free (list);
-          }
+          gst_element_found_tags_for_pad (GST_ELEMENT (mad), mad->srcpad, 0,
+              list);
         }
 
         mad->check_for_xing = FALSE;
@@ -1395,8 +1424,10 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       }
 
       /* if we're not resyncing/in error, check if caps need to be set again */
-      if (!mad->in_error)
-        gst_mad_check_caps_reset (mad);
+      if (!mad->in_error) {
+        if (!gst_mad_check_caps_reset (mad))
+          goto end;
+      }
       nsamples = MAD_NSBSAMPLES (&mad->frame.header) *
           (mad->stream.options & MAD_OPTION_HALFSAMPLERATE ? 16 : 32);
 
@@ -1409,9 +1440,11 @@ gst_mad_chain (GstPad * pad, GstData * _data)
         /* if we have a pending timestamp, we can use it now to calculate the sample offset */
         if (GST_CLOCK_TIME_IS_VALID (mad->last_ts)) {
           GstFormat format = GST_FORMAT_DEFAULT;
+          gint64 total_samples;
 
           gst_pad_convert (mad->srcpad, GST_FORMAT_TIME, mad->last_ts, &format,
-              &mad->total_samples);
+              &total_samples);
+          mad->total_samples = total_samples;
           mad->last_ts = GST_CLOCK_TIME_NONE;
         }
         time_offset = mad->total_samples * GST_SECOND / mad->rate;
@@ -1463,6 +1496,8 @@ gst_mad_chain (GstPad * pad, GstData * _data)
           }
         }
 
+        GST_DEBUG ("Pushing buf with ts %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (outbuffer)));
         gst_pad_push (mad->srcpad, GST_DATA (outbuffer));
       }
 
@@ -1489,11 +1524,14 @@ gst_mad_chain (GstPad * pad, GstData * _data)
       if (consumed == 0)
         consumed = mad->stream.next_frame - mad_input_buffer;
 
+      if (mad->stream.skiplen > consumed)
+        consumed = mad->stream.skiplen;
       GST_LOG ("mad consumed %d bytes", consumed);
       /* move out pointer to where mad want the next data */
       mad_input_buffer += consumed;
       mad->tempsize -= consumed;
       mad->bytes_consumed += consumed;
+      mad->stream.skiplen = 0;
     }
     /* we only get here from breaks, tempsize never actually drops below 0 */
     memmove (mad->tempbuffer, mad_input_buffer, mad->tempsize);
