@@ -33,12 +33,6 @@
 #define NAS_SOUND_PORT_DURATION	(2)
 
 GST_DEBUG_CATEGORY (NAS);
-/* Signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
 
 enum
 {
@@ -51,7 +45,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("audio/x-raw-int, "
-        "endianess = (int) BYTE_ORDER, "
+        "endianness = (int) BYTE_ORDER, "
         "signed = (boolean) TRUE, "
         "width = (int) 16, "
         "depth = (int) 16, "
@@ -67,7 +61,7 @@ static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink",
 static void gst_nassink_base_init (gpointer g_class);
 static void gst_nassink_class_init (GstNassinkClass * klass);
 static void gst_nassink_init (GstNassink * nassink);
-static void gst_nassink_dispose (GObject * object);
+static void gst_nassink_finalize (GObject * object);
 
 static gboolean gst_nassink_open_audio (GstNassink * sink);
 static void gst_nassink_close_audio (GstNassink * sink);
@@ -76,6 +70,7 @@ static GstCaps *gst_nassink_getcaps (GstPad * pad);
 static gboolean gst_nassink_sync_parms (GstNassink * nassink);
 static GstPadLinkReturn gst_nassink_sinkconnect (GstPad * pad,
     const GstCaps * caps);
+static GstCaps *gst_nassink_fixate (GstPad * pad, const GstCaps * caps);
 
 static void gst_nassink_chain (GstPad * pad, GstData * _data);
 
@@ -153,7 +148,7 @@ gst_nassink_class_init (GstNassinkClass * klass)
 
   gobject_class->set_property = gst_nassink_set_property;
   gobject_class->get_property = gst_nassink_get_property;
-  gobject_class->dispose = gst_nassink_dispose;
+  gobject_class->finalize = gst_nassink_finalize;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MUTE, g_param_spec_boolean ("mute", "mute", "mute", TRUE, G_PARAM_READWRITE));   /* CHECKME */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_HOST, g_param_spec_string ("host", "host", "host", NULL, G_PARAM_READWRITE));    /* CHECKME */
@@ -173,6 +168,9 @@ gst_nassink_init (GstNassink * nassink)
       GST_DEBUG_FUNCPTR (gst_nassink_chain));
   gst_pad_set_link_function (nassink->sinkpad, gst_nassink_sinkconnect);
   gst_pad_set_getcaps_function (nassink->sinkpad, gst_nassink_getcaps);
+  gst_pad_set_fixate_function (nassink->sinkpad, gst_nassink_fixate);
+
+  GST_FLAG_SET (nassink, GST_ELEMENT_EVENT_AWARE);
 
   nassink->mute = FALSE;
   nassink->depth = 16;
@@ -190,11 +188,12 @@ gst_nassink_init (GstNassink * nassink)
 }
 
 static void
-gst_nassink_dispose (GObject * object)
+gst_nassink_finalize (GObject * object)
 {
   GstNassink *nassink = GST_NASSINK (object);
 
   g_free (nassink->host);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -204,17 +203,16 @@ gst_nassink_getcaps (GstPad * pad)
   GstCaps *templatecaps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
   GstCaps *caps;
   int i;
-  AuServer *server;
 
-  server = AuOpenServer (nassink->host, 0, NULL, 0, NULL, NULL);
-  if (!server)
+  if (!nassink->audio)
     return templatecaps;
 
   for (i = 0; i < gst_caps_get_size (templatecaps); i++) {
     GstStructure *structure = gst_caps_get_structure (templatecaps, i);
 
     gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE,
-        AuServerMinSampleRate (server), AuServerMaxSampleRate (server), NULL);
+        AuServerMinSampleRate (nassink->audio),
+        AuServerMaxSampleRate (nassink->audio), NULL);
   }
   caps = gst_caps_intersect (templatecaps, gst_pad_get_pad_template_caps (pad));
   gst_caps_free (templatecaps);
@@ -235,7 +233,9 @@ gst_nassink_sync_parms (GstNassink * nassink)
   if (nassink->audio == NULL)
     return TRUE;
 
-  GST_CAT_DEBUG (NAS, "depth=%i rate=%i", nassink->depth, nassink->rate);
+  GST_CAT_DEBUG (NAS, "depth=%i rate=%i channels=%i", nassink->depth,
+      nassink->rate, nassink->tracks);
+
   if (nassink->flow != AuNone) {
     GST_CAT_DEBUG (NAS, "flushing buffer");
     while (nassink->pos && nassink->buf)
@@ -259,6 +259,83 @@ gst_nassink_sync_parms (GstNassink * nassink)
   return ret >= 0;
 }
 
+/* nicked from gstalsa.c */
+static GstCaps *
+gst_nassink_fixate_field_nearest_int (const GstCaps * caps,
+    const gchar * field_name, gint target)
+{
+  guint i;
+  GstCaps *result;
+  GstCaps *smaller = gst_caps_new_empty ();
+  GstCaps *equal = gst_caps_new_empty ();
+  GstCaps *bigger = gst_caps_new_empty ();
+
+  /* works like this: we fixate every structure and put them into one of those 
+   * caps depending on what we fixated to. We then return the best caps that is 
+   * not empty in the following order: equal, bigger, smaller 
+   * We also make sure the caps were really reduced.
+   */
+  for (i = 0; i < gst_caps_get_size (caps); i++) {
+    gint fixated_to;
+    GstStructure *copy = gst_structure_copy (gst_caps_get_structure (caps, i));
+
+    gst_caps_structure_fixate_field_nearest_int (copy, field_name, target);
+    if (gst_structure_get_int (copy, field_name, &fixated_to)) {
+      if (fixated_to == target) {
+        gst_caps_append_structure (equal, copy);
+      } else if (fixated_to > target) {
+        gst_caps_append_structure (bigger, copy);
+      } else {
+        gst_caps_append_structure (smaller, copy);
+      }
+    } else {
+      /* FIXME: what do we do here? Add to all or throw an error? */
+      g_return_val_if_reached (NULL);
+    }
+  }
+  if (!gst_caps_is_empty (equal)) {
+    gst_caps_free (bigger);
+    gst_caps_free (smaller);
+    result = equal;
+  } else {
+    gst_caps_free (equal);
+    if (!gst_caps_is_empty (bigger)) {
+      gst_caps_free (smaller);
+      result = bigger;
+    } else {
+      gst_caps_free (bigger);
+      if (gst_caps_is_empty (smaller)) {
+        gst_caps_free (smaller);
+        return NULL;
+      }
+      result = smaller;
+    }
+  }
+  if (gst_caps_is_subset (caps, result)) {
+    /* we didn't reduce caps */
+    gst_caps_free (result);
+    return NULL;
+  }
+  return result;
+}
+
+static GstCaps *
+gst_nassink_fixate (GstPad * pad, const GstCaps * caps)
+{
+  GstCaps *ret;
+
+  if ((ret = gst_nassink_fixate_field_nearest_int (caps, "rate", 44100)))
+    return ret;
+  if ((ret = gst_nassink_fixate_field_nearest_int (caps, "channels", 2)))
+    return ret;
+  if ((ret = gst_nassink_fixate_field_nearest_int (caps, "width", 16)))
+    return ret;
+  if ((ret = gst_nassink_fixate_field_nearest_int (caps, "depth", 16)))
+    return ret;
+
+  return NULL;
+}
+
 static GstPadLinkReturn
 gst_nassink_sinkconnect (GstPad * pad, const GstCaps * caps)
 {
@@ -269,9 +346,10 @@ gst_nassink_sinkconnect (GstPad * pad, const GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
 
-  gst_structure_get_int (structure, "depth", &nassink->depth);
-  gst_structure_get_int (structure, "channels", &nassink->tracks);
-  gst_structure_get_int (structure, "rate", &nassink->rate);
+  if (!gst_structure_get_int (structure, "depth", &nassink->depth)
+      || !gst_structure_get_int (structure, "channels", &nassink->tracks)
+      || !gst_structure_get_int (structure, "rate", &nassink->rate))
+    return GST_PAD_LINK_REFUSED;
 
   if (!gst_nassink_sync_parms (nassink))
     return GST_PAD_LINK_REFUSED;
@@ -279,10 +357,42 @@ gst_nassink_sinkconnect (GstPad * pad, const GstCaps * caps)
   return GST_PAD_LINK_OK;
 }
 
+static void
+gst_nassink_chain_handle_event (GstNassink * nassink, GstPad * pad,
+    GstEvent * event)
+{
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_EOS:
+/*
+      gst_audio_clock_set_active (GST_AUDIO_CLOCK (esdsink->provided_clock),
+          FALSE);
+*/
+      gst_pad_event_default (pad, event);
+      return;
+    case GST_EVENT_DISCONTINUOUS:
+      /* evil hack */
+      /*
+         gst_nassink_close_audio (nassink);
+         gst_nassink_open_audio (nassink);
+         nassink->flow = AuGetScratchFlow (nassink->audio, NULL);
+         if (nassink->flow == 0) {
+         GST_CAT_DEBUG (NAS, "couldn't get flow");
+         // return -1 (throw error?)
+         }
+       */
+      gst_pad_event_default (pad, event);
+      return;
+    default:
+      gst_pad_event_default (pad, event);
+      return;
+  }
+  gst_event_unref (event);
+}
+
 static void
 gst_nassink_chain (GstPad * pad, GstData * _data)
 {
-  GstBuffer *buf = GST_BUFFER (_data);
+  GstBuffer *buf;
   int pos = 0;
   int remaining;
   int available;
@@ -290,12 +400,21 @@ gst_nassink_chain (GstPad * pad, GstData * _data)
 
   g_return_if_fail (pad != NULL);
   g_return_if_fail (GST_IS_PAD (pad));
-  g_return_if_fail (buf != NULL);
+  g_return_if_fail (_data != NULL);
 
   nassink = GST_NASSINK (gst_pad_get_parent (pad));
 
+  if (GST_IS_EVENT (_data)) {
+    gst_nassink_chain_handle_event (nassink, pad, GST_EVENT (_data));
+    return;
+  }
+
+  if (nassink->buf == NULL)
+    gst_nassink_sync_parms (nassink);   // FIXME: if return false, error
+
   g_return_if_fail (nassink->buf != NULL);
 
+  buf = GST_BUFFER (_data);
   if (GST_BUFFER_DATA (buf) != NULL) {
     if (!nassink->mute && nassink->audio != NULL) {
 
@@ -409,11 +528,17 @@ gst_nassink_open_audio (GstNassink * sink)
   /* Open Server */
 
   sink->audio = AuOpenServer (sink->host, 0, NULL, 0, NULL, NULL);
-  if (sink->audio == NULL)
+  if (sink->audio == NULL) {
+    GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
+        ("can't open connection to nas server"));
     return FALSE;
+  }
   sink->device = NAS_getDevice (sink->audio, sink->tracks);
   if (sink->device == AuNone) {
-    GST_CAT_DEBUG (NAS, "no device with %i tracks found", sink->tracks);
+    AuCloseServer (sink->audio);
+    sink->audio = NULL;
+    GST_ELEMENT_ERROR (sink, RESOURCE, OPEN_WRITE, (NULL),
+        ("can't get nas device with %i tracks", sink->tracks));
     return FALSE;
   }
 
@@ -422,10 +547,6 @@ gst_nassink_open_audio (GstNassink * sink)
   sink->pos = 0;
   sink->buf = NULL;
 
-  /* Start a flow */
-
-  GST_FLAG_SET (sink, GST_NASSINK_OPEN);
-
   GST_CAT_DEBUG (NAS, "opened audio device");
   return TRUE;
 }
@@ -454,8 +575,6 @@ gst_nassink_close_audio (GstNassink * sink)
   AuCloseServer (sink->audio);
   sink->audio = NULL;
 
-  GST_FLAG_UNSET (sink, GST_NASSINK_OPEN);
-
   GST_CAT_DEBUG (NAS, "closed audio device");
 }
 
@@ -470,21 +589,26 @@ gst_nassink_change_state (GstElement * element)
 
   switch (GST_STATE_PENDING (element)) {
     case GST_STATE_NULL:
-      if (GST_FLAG_IS_SET (element, GST_NASSINK_OPEN))
-        gst_nassink_close_audio (nassink);
+      gst_nassink_close_audio (nassink);
       break;
 
     case GST_STATE_READY:
-      if (!GST_FLAG_IS_SET (element, GST_NASSINK_OPEN))
-        gst_nassink_open_audio (nassink);
+      if (!gst_nassink_open_audio (nassink))
+        return GST_STATE_FAILURE;
       break;
 
     case GST_STATE_PAUSED:
-      while (nassink->pos && nassink->buf)
-        NAS_flush (nassink);
+      if (nassink->flow != AuNone)
+        AuPauseFlow (nassink->audio, nassink->flow, NULL);
+      /*
+         while (nassink->pos && nassink->buf)
+         NAS_flush (nassink);
+       */
       break;
 
     case GST_STATE_PLAYING:
+      if (nassink->flow != AuNone)
+        AuStartFlow (nassink->audio, nassink->flow, NULL);
       break;
   }
 
@@ -682,6 +806,14 @@ NAS_createFlow (GstNassink * sink, unsigned char format, unsigned short rate,
     }
   }
 
+  /* needed to change number of channels */
+  sink->device = NAS_getDevice (sink->audio, sink->tracks);
+  if (sink->device == AuNone) {
+    GST_CAT_DEBUG (NAS, "no device with %i tracks found", sink->tracks);
+    return -1;
+  }
+
+
   buf_samples = rate * NAS_SOUND_PORT_DURATION;
 
 
