@@ -80,6 +80,7 @@ typedef struct
   gint64 packetno;              /* number of next expected packet */
 
   guint64 start;                /* first valid granulepos */
+  guint64 start_time;           /* cache conversion to time units */
   guint64 length;               /* length of stream or 0 */
   glong pages;                  /* number of pages in stream or 0 */
 
@@ -406,7 +407,7 @@ get_relative (GstOggDemux * ogg, GstOggPad * cur, gint64 granpos, GstFormat out)
   GstFormat fmt;
 
   /* we're gonna ask our peer */
-  if (!GST_PAD_PEER (cur->pad))
+  if (!cur->pad || !GST_PAD_PEER (cur->pad))
     return -1;
 
   /* lineair unit (time) */
@@ -417,12 +418,13 @@ get_relative (GstOggDemux * ogg, GstOggPad * cur, gint64 granpos, GstFormat out)
 
   /* get base for this chain */
   FOR_PAD_IN_CURRENT_CHAIN (ogg, pad,
-      if (pad->start != -1 &&
-          GST_PAD_PEER (pad->pad) &&
+      if (pad->start_time == -1 && pad->start != -1 &&
+          pad->pad && GST_PAD_PEER (pad->pad) &&
           gst_pad_convert (GST_PAD_PEER (pad->pad),
-              GST_FORMAT_DEFAULT, pad->start,
-              &fmt, &tmp) && (start == -1 || tmp < start))
-      start = tmp;);
+              GST_FORMAT_DEFAULT, pad->start, &fmt, &tmp)) {
+      pad->start_time = tmp;}
+      if (pad->start_time != -1 && (start == -1 || pad->start_time < start))
+      start = pad->start_time;);
   if (start == -1)
     return -1;
 
@@ -774,13 +776,13 @@ gst_ogg_demux_position (GstOggDemux * ogg)
 static void
 gst_ogg_pad_populate (GstOggDemux * ogg, GstOggPad * pad, ogg_page * page)
 {
-  gint64 start, end;
+  gint64 start, end, granpos = ogg_page_granulepos (page);
 
-  if (pad->start > ogg_page_granulepos (page) && ogg_page_granulepos (page) > 0) {
-    pad->start = ogg_page_granulepos (page);
-  }
-  if (pad->length < ogg_page_granulepos (page))
-    pad->length = ogg_page_granulepos (page);
+  if (pad->start > granpos && granpos > 0 && ogg->setup_state <= SETUP_READ_BOS)
+    pad->start = granpos;
+  if (pad->length < granpos && granpos > 0 &&
+      ogg->setup_state >= SETUP_FIND_LAST_CHAIN)
+    pad->length = granpos;
   if (pad->pages < ogg_page_pageno (page))
     pad->pages = ogg_page_pageno (page);
   end = gst_ogg_demux_position (ogg);
@@ -1301,8 +1303,9 @@ gst_ogg_pad_new (GstOggDemux * ogg, int serial)
   GST_LOG_OBJECT (ogg, "created new ogg src %p for stream with serial %d", ret,
       serial);
   ret->start_offset = ret->end_offset = -1;
-  ret->start = -1;
+  ret->start = ret->start_time = -1;
   ret->start_found = ret->end_found = FALSE;
+  ret->offset = GST_BUFFER_OFFSET_NONE;
 
   return ret;
 }
@@ -1364,8 +1367,8 @@ gst_ogg_demux_push (GstOggDemux * ogg, ogg_page * page)
     ogg->bos = FALSE;
   }
   if (cur == NULL) {
-    GST_ELEMENT_ERROR (ogg, STREAM, DECODE, (NULL),
-        ("invalid ogg stream serial no"));
+    /* not fatal */
+    GST_ERROR_OBJECT (ogg, "invalid ogg stream serial no (broken stream?)");
     return;
   }
   if (ogg_stream_pagein (&cur->stream, page) != 0) {
@@ -1382,6 +1385,7 @@ gst_ogg_demux_push (GstOggDemux * ogg, ogg_page * page)
           cur->known_offset, cur->serial, ogg->seek_to, ogg->seek_pad->serial);
 
       if (cur != ogg->seek_pad) {
+        ogg_stream_reset (&cur->stream);
         break;
       } else {
         gint64 position, diff;
@@ -1404,6 +1408,7 @@ gst_ogg_demux_push (GstOggDemux * ogg, ogg_page * page)
         if (diff < GST_SECOND) {
           GST_DEBUG ("Close enough (%" GST_TIME_FORMAT " seconds off)",
               GST_TIME_ARGS (diff));
+          ogg->seek_to = position;
           goto play;
         }
 
@@ -1422,6 +1427,7 @@ gst_ogg_demux_push (GstOggDemux * ogg, ogg_page * page)
           goto play;
         }
         ogg->seek_try++;
+        ogg_stream_reset (&ogg->seek_pad->stream);
         ogg_sync_clear (&ogg->sync);
         return;
 
@@ -1430,6 +1436,38 @@ gst_ogg_demux_push (GstOggDemux * ogg, ogg_page * page)
         GST_DEBUG_OBJECT (ogg,
             "ended seek at offset %" G_GUINT64_FORMAT " (requested  %"
             G_GUINT64_FORMAT, cur->known_offset, ogg->seek_to);
+
+        FOR_PAD_IN_CURRENT_CHAIN (ogg, pad,
+            if ((pad->flags & GST_OGG_PAD_NEEDS_FLUSH)
+                && GST_PAD_IS_USABLE (pad->pad)) {
+            gst_pad_push (pad->pad,
+                  GST_DATA (gst_event_new (GST_EVENT_FLUSH)));
+              pad->flags &= (~GST_OGG_PAD_NEEDS_FLUSH);}
+
+            /* send discont if needed */
+            if ((pad->flags & GST_OGG_PAD_NEEDS_DISCONT)
+                && GST_PAD_IS_USABLE (pad->pad)) {
+              /* so in order to synchronized the variety of streams, we will
+               * not use the granpos but the last seekpos for the discont. */
+              GstFormat fmt = GST_FORMAT_DEFAULT;
+              GstEvent * event;
+              gint64 discont;
+              if (!gst_pad_convert (GST_PAD_PEER (pad->pad),
+                      ogg->seek_format, ogg->seek_to, &fmt, &discont)) {
+              event = gst_event_new_discontinuous (FALSE,
+                    ogg->seek_format, ogg->seek_to, GST_FORMAT_UNDEFINED);}
+              else {
+              event = gst_event_new_discontinuous (FALSE,
+                    GST_FORMAT_DEFAULT, discont, GST_FORMAT_UNDEFINED);
+                pad->offset = discont;}
+
+              /* FIXME: this might be wrong because we can only use the last
+               * known offset */
+
+            gst_pad_push (pad->pad, GST_DATA (event));
+              pad->flags &= (~GST_OGG_PAD_NEEDS_DISCONT);}
+        );
+
         ogg->seek_pad = NULL;
         ogg->seek_offset = 0;
         ogg->seek_try = 0;
@@ -1455,6 +1493,38 @@ gst_ogg_demux_push (GstOggDemux * ogg, ogg_page * page)
     cur->pad = NULL;
   }
 }
+
+static void
+gst_ogg_sync (GstOggDemux * ogg, GstOggPad * cur)
+{
+  gint64 bias, time;
+  GstFormat fmt = GST_FORMAT_TIME;
+
+  if ((time = get_relative (ogg, cur, cur->offset, GST_FORMAT_TIME)) == -1)
+    return;
+  FOR_PAD_IN_CURRENT_CHAIN (ogg, pad,
+      if (pad->pad && GST_PAD_PEER (pad->pad) &&
+          pad->offset != GST_BUFFER_OFFSET_NONE) {
+        if (gst_pad_query (GST_PAD_PEER (pad->pad),
+                GST_QUERY_POSITION, &fmt, &bias) && bias + GST_SECOND < time) {
+          GstEvent * event;
+          gint64 val = 0;
+          GstFormat fmt = GST_FORMAT_DEFAULT;
+          event = gst_event_new_filler_stamped (bias,
+              time - bias - GST_SECOND / 2);
+          GST_DEBUG ("Syncing stream %d at time %" GST_TIME_FORMAT
+              " and duration %" GST_TIME_FORMAT,
+              pad->serial, GST_TIME_ARGS (bias),
+              GST_TIME_ARGS (time - bias - GST_SECOND / 2));
+          gst_pad_push (pad->pad, GST_DATA (event));
+          gst_pad_convert (GST_PAD_PEER (pad->pad),
+              GST_FORMAT_TIME, bias, &fmt, &val);
+          /* hmm... */
+        pad->offset = pad->start + val;}
+      }
+  );
+}
+
 static void
 gst_ogg_pad_push (GstOggDemux * ogg, GstOggPad * pad)
 {
@@ -1523,44 +1593,6 @@ gst_ogg_pad_push (GstOggDemux * ogg, GstOggPad * pad)
           pos = get_relative (ogg, pad, pad->known_offset, GST_FORMAT_DEFAULT);
         }
 
-        if ((pad->flags & GST_OGG_PAD_NEEDS_FLUSH)
-            && GST_PAD_IS_USABLE (pad->pad)) {
-          gst_pad_push (pad->pad, GST_DATA (gst_event_new (GST_EVENT_FLUSH)));
-          pad->flags &= (~GST_OGG_PAD_NEEDS_FLUSH);
-        }
-
-        /* send discont if needed */
-        if ((pad->flags & GST_OGG_PAD_NEEDS_DISCONT)
-            && GST_PAD_IS_USABLE (pad->pad)) {
-          /* so in order to synchronized the variety of streams, we will
-           * not use the granpos but the last seekpos for the discont. */
-          GstFormat fmt;
-          GstEvent *event;
-          gint64 discont;
-
-          if (pos != -1) {
-            fmt = GST_FORMAT_DEFAULT;
-            if (!GST_PAD_PEER (pad->pad) ||
-                !gst_pad_convert (GST_PAD_PEER (pad->pad),
-                    ogg->seek_format, ogg->seek_to, &fmt, &discont)) {
-              event = gst_event_new_discontinuous (FALSE,
-                  ogg->seek_format, ogg->seek_to, GST_FORMAT_UNDEFINED);
-            } else {
-              event = gst_event_new_discontinuous (FALSE,
-                  GST_FORMAT_DEFAULT, discont, GST_FORMAT_UNDEFINED);
-            }
-          } else {
-            event = gst_event_new_discontinuous (FALSE,
-                GST_FORMAT_DEFAULT, 0, GST_FORMAT_UNDEFINED);
-          }
-
-          /* FIXME: this might be wrong because we can only use the last
-           * known offset */
-
-          gst_pad_push (pad->pad, GST_DATA (event));
-          pad->flags &= (~GST_OGG_PAD_NEEDS_DISCONT);
-        }
-
         /* optimization: use a bufferpool containing the ogg packet? */
         buf =
             gst_pad_alloc_buffer (pad->pad, GST_BUFFER_OFFSET_NONE,
@@ -1571,8 +1603,11 @@ gst_ogg_pad_push (GstOggDemux * ogg, GstOggPad * pad)
         if (packet.granulepos != -1 && pos != -1)
           GST_BUFFER_OFFSET_END (buf) = pos;
         pad->offset = packet.granulepos;
+        gst_ogg_sync (ogg, pad);
         if (GST_PAD_IS_USABLE (pad->pad))
           gst_pad_push (pad->pad, GST_DATA (buf));
+        else
+          gst_buffer_unref (buf);
         break;
       }
       default:
@@ -1693,6 +1728,7 @@ gst_ogg_type_find (ogg_packet * packet)
   gst_find.data = &find;
   gst_find.peek = ogg_find_peek;
   gst_find.suggest = ogg_find_suggest;
+  gst_find.get_length = NULL;
 
   while (walk) {
     GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (walk->data);
