@@ -39,6 +39,10 @@ GST_DEBUG_CATEGORY_STATIC (gst_ogm_parse_debug);
 #define GST_IS_OGM_AUDIO_PARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj), GST_TYPE_OGM_AUDIO_PARSE))
 
+#define GST_TYPE_OGM_TEXT_PARSE (gst_ogm_text_parse_get_type())
+#define GST_IS_OGM_TEXT_PARSE(obj) \
+  (G_TYPE_CHECK_INSTANCE_TYPE((obj), GST_TYPE_OGM_TEXT_PARSE))
+
 #define GST_TYPE_OGM_PARSE (gst_ogm_parse_get_type())
 #define GST_OGM_PARSE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST((obj), GST_TYPE_OGM_PARSE, GstOgmParse))
@@ -87,6 +91,7 @@ typedef struct _stream_header
   {
     stream_header_video video;
     stream_header_audio audio;
+    /* text has no additional data */
   } s;
 } stream_header;
 
@@ -116,28 +121,37 @@ GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
 static GstStaticPadTemplate ogm_audio_parse_sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-ogm-audio"));
-static GstPadTemplate *video_src_templ, *audio_src_templ;
+static GstStaticPadTemplate ogm_text_parse_sink_template_factory =
+GST_STATIC_PAD_TEMPLATE ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
+    GST_STATIC_CAPS ("application/x-ogm-text"));
+static GstPadTemplate *video_src_templ, *audio_src_templ, *text_src_templ;
 
 static GType gst_ogm_audio_parse_get_type (void);
 static GType gst_ogm_video_parse_get_type (void);
+static GType gst_ogm_text_parse_get_type (void);
 static GType gst_ogm_parse_get_type (void);
 
 static void gst_ogm_audio_parse_base_init (GstOgmParseClass * klass);
 static void gst_ogm_video_parse_base_init (GstOgmParseClass * klass);
+static void gst_ogm_text_parse_base_init (GstOgmParseClass * klass);
 static void gst_ogm_parse_class_init (GstOgmParseClass * klass);
 static void gst_ogm_parse_init (GstOgmParse * ogm);
 static void gst_ogm_video_parse_init (GstOgmParse * ogm);
 static void gst_ogm_audio_parse_init (GstOgmParse * ogm);
+static void gst_ogm_text_parse_init (GstOgmParse * ogm);
 
 static const GstFormat *gst_ogm_parse_get_sink_formats (GstPad * pad);
+static const GstQueryType *gst_ogm_parse_get_sink_querytypes (GstPad * pad);
 static gboolean gst_ogm_parse_sink_convert (GstPad * pad, GstFormat src_format,
     gint64 src_value, GstFormat * dest_format, gint64 * dest_value);
+static gboolean gst_ogm_parse_sink_query (GstPad * pad, GstQueryType type,
+    GstFormat * fmt, gint64 * val);
 
 static void gst_ogm_parse_chain (GstPad * pad, GstData * data);
 
 static GstElementStateReturn gst_ogm_parse_change_state (GstElement * element);
 
-GstElementClass *parent_class = NULL;
+static GstElementClass *parent_class = NULL;
 
 static GType
 gst_ogm_parse_get_type (void)
@@ -217,6 +231,32 @@ gst_ogm_video_parse_get_type (void)
   return ogm_video_parse_type;
 }
 
+GType
+gst_ogm_text_parse_get_type (void)
+{
+  static GType ogm_text_parse_type = 0;
+
+  if (!ogm_text_parse_type) {
+    static const GTypeInfo ogm_text_parse_info = {
+      sizeof (GstOgmParseClass),
+      (GBaseInitFunc) gst_ogm_text_parse_base_init,
+      NULL,
+      NULL,
+      NULL,
+      NULL,
+      sizeof (GstOgmParse),
+      0,
+      (GInstanceInitFunc) gst_ogm_text_parse_init,
+    };
+
+    ogm_text_parse_type =
+        g_type_register_static (GST_TYPE_OGM_PARSE,
+        "GstOgmTextParse", &ogm_text_parse_info, 0);
+  }
+
+  return ogm_text_parse_type;
+}
+
 static void
 gst_ogm_audio_parse_base_init (GstOgmParseClass * klass)
 {
@@ -257,6 +297,26 @@ gst_ogm_video_parse_base_init (GstOgmParseClass * klass)
   gst_element_class_add_pad_template (element_class, video_src_templ);
 }
 
+static void
+gst_ogm_text_parse_base_init (GstOgmParseClass * klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+  static GstElementDetails gst_ogm_text_parse_details =
+      GST_ELEMENT_DETAILS ("OGM text stream parser",
+      "Codec/Decoder/Subtitle",
+      "parse an OGM text header and stream",
+      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+  GstCaps *caps = gst_caps_new_simple ("text/plain", NULL);
+
+  gst_element_class_set_details (element_class, &gst_ogm_text_parse_details);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&ogm_text_parse_sink_template_factory));
+  text_src_templ = gst_pad_template_new ("src",
+      GST_PAD_SRC, GST_PAD_SOMETIMES, caps);
+  gst_element_class_add_pad_template (element_class, text_src_templ);
+}
+
 static void
 gst_ogm_parse_class_init (GstOgmParseClass * klass)
 {
@@ -273,6 +333,8 @@ gst_ogm_parse_init (GstOgmParse * ogm)
   /* initalize */
   memset (&ogm->hdr, 0, sizeof (ogm->hdr));
   ogm->next_granulepos = 0;
+
+  GST_FLAG_SET (ogm, GST_ELEMENT_EVENT_AWARE);
 }
 
 static void
@@ -317,6 +379,30 @@ gst_ogm_video_parse_init (GstOgmParse * ogm)
   ogm->srcpadtempl = video_src_templ;
 }
 
+static void
+gst_ogm_text_parse_init (GstOgmParse * ogm)
+{
+  GstPadTemplate *templ;
+
+  /* create the pads */
+  templ = gst_static_pad_template_get (&ogm_text_parse_sink_template_factory);
+  ogm->sinkpad = gst_pad_new_from_template (templ, "sink");
+  gst_pad_set_convert_function (ogm->sinkpad, gst_ogm_parse_sink_convert);
+  gst_pad_set_formats_function (ogm->sinkpad, gst_ogm_parse_get_sink_formats);
+  gst_pad_set_query_type_function (ogm->sinkpad,
+      gst_ogm_parse_get_sink_querytypes);
+  gst_pad_set_query_function (ogm->sinkpad, gst_ogm_parse_sink_query);
+  gst_pad_set_chain_function (ogm->sinkpad, gst_ogm_parse_chain);
+  gst_element_add_pad (GST_ELEMENT (ogm), ogm->sinkpad);
+
+#if 0
+  ogm->srcpad = gst_pad_new_from_template (text_src_templ, "src");
+  gst_pad_use_explicit_caps (ogm->srcpad);
+  gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
+#endif
+  ogm->srcpadtempl = text_src_templ;
+}
+
 static const GstFormat *
 gst_ogm_parse_get_sink_formats (GstPad * pad)
 {
@@ -329,6 +415,17 @@ gst_ogm_parse_get_sink_formats (GstPad * pad)
   return formats;
 }
 
+static const GstQueryType *
+gst_ogm_parse_get_sink_querytypes (GstPad * pad)
+{
+  static const GstQueryType types[] = {
+    GST_QUERY_POSITION,
+    0
+  };
+
+  return types;
+}
+
 static gboolean
 gst_ogm_parse_sink_convert (GstPad * pad,
     GstFormat src_format, gint64 src_value,
@@ -347,6 +444,7 @@ gst_ogm_parse_sink_convert (GstPad * pad,
               res = TRUE;
               break;
             case 'v':
+            case 't':
               *dest_value = (GST_SECOND / 10000000) *
                   ogm->hdr.time_unit * src_value;
               res = TRUE;
@@ -359,6 +457,28 @@ gst_ogm_parse_sink_convert (GstPad * pad,
           break;
       }
       break;
+    case GST_FORMAT_TIME:
+      switch (*dest_format) {
+        case GST_FORMAT_DEFAULT:
+          switch (ogm->hdr.streamtype[0]) {
+            case 'a':
+              *dest_value = ogm->hdr.samples_per_unit * src_value / GST_SECOND;
+              res = TRUE;
+              break;
+            case 'v':
+            case 't':
+              *dest_value = src_value /
+                  ((GST_SECOND / 10000000) * ogm->hdr.time_unit);
+              res = TRUE;
+              break;
+            default:
+              break;
+          }
+          break;
+        default:
+          break;
+      }
+      break;
     default:
       break;
   }
@@ -366,19 +486,68 @@ gst_ogm_parse_sink_convert (GstPad * pad,
   return res;
 }
 
+static gboolean
+gst_ogm_parse_sink_query (GstPad * pad,
+    GstQueryType type, GstFormat * fmt, gint64 * val)
+{
+  GstOgmParse *ogm = GST_OGM_PARSE (gst_pad_get_parent (pad));
+
+  if (type != GST_QUERY_POSITION)
+    return FALSE;
+  if (*fmt != GST_FORMAT_DEFAULT && *fmt != GST_FORMAT_TIME)
+    return FALSE;
+
+  return gst_pad_convert (pad,
+      GST_FORMAT_DEFAULT, ogm->next_granulepos, fmt, val);
+}
+
+static void
+gst_ogm_parse_kick_event (GstOgmParse * ogm, GstEvent * event)
+{
+  switch (GST_EVENT_TYPE (event)) {
+    case GST_EVENT_DISCONTINUOUS:{
+      gint64 val, tval;
+      GstFormat fmt = GST_FORMAT_TIME;
+
+      if (gst_event_discont_get_value (event, GST_FORMAT_DEFAULT, &val) &&
+          gst_pad_convert (ogm->sinkpad, GST_FORMAT_DEFAULT, val,
+              &fmt, &tval)) {
+        gst_event_unref (event);
+        ogm->next_granulepos = val;
+        event = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, tval,
+            GST_FORMAT_UNDEFINED);
+      }
+      /* fall-through */
+    }
+    default:
+      gst_pad_event_default (ogm->sinkpad, event);
+      break;
+  }
+}
+
 static void
 gst_ogm_parse_chain (GstPad * pad, GstData * dat)
 {
   GstOgmParse *ogm = GST_OGM_PARSE (gst_pad_get_parent (pad));
-  GstBuffer *buf = GST_BUFFER (dat);
-  guint8 *data = GST_BUFFER_DATA (buf);
-  guint size = GST_BUFFER_SIZE (buf);
+  GstBuffer *buf;
+  guint8 *data;
+  guint size;
+
+  if (GST_IS_EVENT (dat)) {
+    gst_ogm_parse_kick_event (ogm, GST_EVENT (dat));
+    return;
+  }
+
+  buf = GST_BUFFER (dat);
+  data = GST_BUFFER_DATA (buf);
+  size = GST_BUFFER_SIZE (buf);
 
   GST_DEBUG_OBJECT (ogm, "New packet with packet start code 0x%02x", data[0]);
 
   switch (data[0]) {
     case 0x01:{
       GstCaps *caps = NULL;
+      gchar *codec_name = NULL;
 
       /* stream header */
       if (size < sizeof (stream_header) + 1) {
@@ -394,6 +563,8 @@ gst_ogm_parse_chain (GstPad * pad, GstData * dat)
         ogm->hdr.s.audio.channels = GST_READ_UINT32_LE (&data[45]);
         ogm->hdr.s.audio.blockalign = GST_READ_UINT32_LE (&data[47]);
         ogm->hdr.s.audio.avgbytespersec = GST_READ_UINT32_LE (&data[49]);
+      } else if (!memcmp (&data[1], "text\000\000\000\000", 8)) {
+        /* nothing here */
       } else {
         GST_ELEMENT_ERROR (ogm, STREAM, WRONG_TYPE,
             ("Unknown stream type"), (NULL));
@@ -416,7 +587,7 @@ gst_ogm_parse_chain (GstPad * pad, GstData * dat)
             caps = NULL;
             break;
           }
-          caps = gst_riff_create_audio_caps (codec_id, NULL, NULL, NULL);
+          caps = gst_riff_create_audio_caps (codec_id, NULL, NULL, &codec_name);
           gst_caps_set_simple (caps,
               "channels", G_TYPE_INT, ogm->hdr.s.audio.channels,
               "rate", G_TYPE_INT, ogm->hdr.samples_per_unit, NULL);
@@ -441,13 +612,20 @@ gst_ogm_parse_chain (GstPad * pad, GstData * dat)
               ogm->hdr.time_unit, 10000000. / ogm->hdr.time_unit,
               ogm->hdr.samples_per_unit, ogm->hdr.default_len,
               ogm->hdr.buffersize, ogm->hdr.bits_per_sample);
-          caps = gst_riff_create_video_caps (fcc, NULL, NULL, NULL);
+          caps = gst_riff_create_video_caps (fcc, NULL, NULL, &codec_name);
           gst_caps_set_simple (caps,
               "width", G_TYPE_INT, ogm->hdr.s.video.width,
               "height", G_TYPE_INT, ogm->hdr.s.video.height,
               "framerate", G_TYPE_DOUBLE, 10000000. / ogm->hdr.time_unit, NULL);
           break;
         }
+        case 't':
+          GST_LOG_OBJECT (ogm, "Type: %s, s/u: %" G_GINT64_FORMAT
+              ", timeunit=%" G_GINT64_FORMAT,
+              ogm->hdr.streamtype, ogm->hdr.samples_per_unit,
+              ogm->hdr.time_unit);
+          caps = gst_caps_new_simple ("text/plain", NULL);
+          break;
         default:
           g_assert_not_reached ();
       }
@@ -461,6 +639,16 @@ gst_ogm_parse_chain (GstPad * pad, GstData * dat)
         break;
       }
       gst_element_add_pad (GST_ELEMENT (ogm), ogm->srcpad);
+      if (codec_name) {
+        GstTagList *list = gst_tag_list_new ();
+
+        gst_tag_list_add (list, GST_TAG_MERGE_REPLACE,
+            (ogm->hdr.streamtype[0] == 'a') ?
+            GST_TAG_AUDIO_CODEC : GST_TAG_VIDEO_CODEC, codec_name, NULL);
+        gst_element_found_tags_for_pad (GST_ELEMENT (ogm),
+            ogm->srcpad, 0, list);
+        g_free (codec_name);
+      }
       break;
     }
     case 0x03:
@@ -491,15 +679,19 @@ gst_ogm_parse_chain (GstPad * pad, GstData * dat)
           ogm->next_granulepos = GST_BUFFER_OFFSET_END (buf);
         }
         switch (ogm->hdr.streamtype[0]) {
-          case 'v':
+          case 't':
+          case 'v':{
+            gint samples = (ogm->hdr.streamtype[0] == 'v') ? 1 : xsize;
+
             if (keyframe)
               GST_BUFFER_FLAG_SET (sbuf, GST_BUFFER_KEY_UNIT);
             GST_BUFFER_TIMESTAMP (sbuf) = (GST_SECOND / 10000000) *
                 ogm->next_granulepos * ogm->hdr.time_unit;
             GST_BUFFER_DURATION (sbuf) = (GST_SECOND / 10000000) *
-                ogm->hdr.time_unit;
-            ogm->next_granulepos++;
+                ogm->hdr.time_unit * samples;
+            ogm->next_granulepos += samples;
             break;
+          }
           case 'a':
             GST_BUFFER_TIMESTAMP (sbuf) = GST_SECOND *
                 ogm->next_granulepos / ogm->hdr.samples_per_unit;
@@ -508,9 +700,13 @@ gst_ogm_parse_chain (GstPad * pad, GstData * dat)
             ogm->next_granulepos += xsize;
             break;
           default:
-            g_assert_not_reached ();
+            gst_buffer_unref (sbuf);
+            sbuf = NULL;
+            GST_ELEMENT_ERROR (ogm, RESOURCE, SYNC, (NULL), (NULL));
+            break;
         }
-        gst_pad_push (ogm->srcpad, GST_DATA (sbuf));
+        if (sbuf)
+          gst_pad_push (ogm->srcpad, GST_DATA (sbuf));
       } else {
         GST_ELEMENT_ERROR (ogm, STREAM, WRONG_TYPE,
             ("Wrong packet startcode 0x%02x", data[0]), (NULL));
@@ -547,9 +743,10 @@ gst_ogm_parse_plugin_init (GstPlugin * plugin)
 {
   GST_DEBUG_CATEGORY_INIT (gst_ogm_parse_debug, "ogmparse", 0, "ogm parser");
 
-  return gst_library_load ("riff") &&
-      gst_element_register (plugin, "ogmaudioparse", GST_RANK_PRIMARY,
+  return gst_element_register (plugin, "ogmaudioparse", GST_RANK_PRIMARY,
       GST_TYPE_OGM_AUDIO_PARSE) &&
       gst_element_register (plugin, "ogmvideoparse", GST_RANK_PRIMARY,
-      GST_TYPE_OGM_VIDEO_PARSE);
+      GST_TYPE_OGM_VIDEO_PARSE) &&
+      gst_element_register (plugin, "ogmtextparse", GST_RANK_PRIMARY,
+      GST_TYPE_OGM_TEXT_PARSE);
 }
