@@ -24,11 +24,9 @@
 
 #include <gst/gst.h>
 #include <swfdec.h>
+#include <swfdec_render.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 #define GST_TYPE_SWFDEC \
   (gst_swfdec_get_type())
@@ -44,7 +42,8 @@ extern "C" {
 typedef struct _GstSwfdec GstSwfdec;
 typedef struct _GstSwfdecClass GstSwfdecClass;
 
-struct _GstSwfdec {
+struct _GstSwfdec
+{
   GstElement element;
 
   /* pads */
@@ -52,13 +51,15 @@ struct _GstSwfdec {
   GstPad *videopad;
   GstPad *audiopad;
 
-  SwfdecDecoder *state;
+  SwfdecDecoder *decoder;
   gboolean closed;
 
-  /* the timestamp of the next frame */
   gboolean first;
   gboolean have_format;
 
+  gboolean send_discont;
+  int seek_frame;
+
   double rate;
   gint64 timestamp;
   gint64 interval;
@@ -70,18 +71,22 @@ struct _GstSwfdec {
   gint height;
   gint64 total_frames;
 
+  double x, y;
+  int button;
+
+  int skip_frames;
+  int skip_index;
 };
 
-struct _GstSwfdecClass {
+struct _GstSwfdecClass
+{
   GstElementClass parent_class;
+
+  void (*embed_url) (GstElement *element, const char *url);
 };
 
-GType gst_swfdec_get_type(void);
-
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+GType gst_swfdec_get_type (void);
 
+G_END_DECLS
 
 #endif /* __GST_SWFDEC_H__ */
