@@ -1,7 +1,7 @@
 #ifndef __MEM_H
 #define __MEM_H
 
-#include <stdint.h>
+#include "_stdint.h"
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
