@@ -378,7 +378,7 @@ theora_dec_src_query (GstPad * pad, GstQueryType query, GstFormat * format,
   gint64 granulepos;
   GstTheoraDec *dec = GST_THEORA_DEC (gst_pad_get_parent (pad));
   GstFormat my_format = GST_FORMAT_DEFAULT;
-  guint64 time;
+  gint64 time;
 
   if (query == GST_QUERY_POSITION) {
     /* this is easy, we can convert a granule position to everything */
@@ -416,7 +416,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:{
-      guint64 value;
+      gint64 value;
       GstEvent *real_seek;
 
       /* we have to ask our peer to seek to time here as we know
@@ -458,7 +458,7 @@ theora_dec_src_event (GstPad * pad, GstEvent * event)
 static void
 theora_dec_event (GstTheoraDec * dec, GstEvent * event)
 {
-  guint64 value, time, bytes;
+  gint64 value, time, bytes;
 
   GST_LOG_OBJECT (dec, "handling event");
   switch (GST_EVENT_TYPE (event)) {
@@ -556,8 +556,10 @@ theora_dec_chain (GstPad * pad, GstData * data)
   packet.b_o_s = (packet.packetno == 0) ? 1 : 0;
   packet.e_o_s = 0;
 
-  GST_DEBUG_OBJECT (dec, "header=%d packetno=%d, outtime=%" GST_TIME_FORMAT,
-      packet.packet[0], packet.packetno, GST_TIME_ARGS (outtime));
+  GST_DEBUG_OBJECT (dec, "outtime=%" GST_TIME_FORMAT " (%"
+      G_GUINT64_FORMAT ") header=0x%02x packetno=%d",
+      GST_TIME_ARGS (outtime), outtime,
+      (gint) packet.packet[0], packet.packetno);
 
   /* switch depending on packet type */
   if (packet.packet[0] & 0x80) {
@@ -577,8 +579,8 @@ theora_dec_chain (GstPad * pad, GstData * data)
     } else if (packet.packetno == 1) {
       gchar *encoder = NULL;
       GstTagList *list =
-          gst_tag_list_from_vorbiscomment_buffer (buf, "\201theora", 7,
-          &encoder);
+          gst_tag_list_from_vorbiscomment_buffer (buf, (guint8 *) "\201theora",
+          7, &encoder);
 
       if (!list) {
         GST_ERROR_OBJECT (dec, "failed to parse tags");
@@ -702,6 +704,9 @@ theora_dec_chain (GstPad * pad, GstData * data)
       if (add_one) {
         outtime = GST_SECOND * theora_granule_time (&dec->state,
             dec->granulepos);
+        GST_DEBUG_OBJECT (dec,
+            "Correcting output time to %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (outtime));
       }
     } else if (dec->need_keyframe) {
       GST_WARNING_OBJECT (dec, "dropping frame because we need a keyframe");
