@@ -35,9 +35,9 @@ static GstPadTemplate *gst_oggvorbisenc_src_template,
 
 /* elementfactory information */
 GstElementDetails oggvorbisenc_details = {
-  "Ogg Vorbis encoder",
+  "Ogg Vorbis encoder (deprecated)",
   "Codec/Encoder/Audio",
-  "Encodes audio in OGG Vorbis format",
+  "Encodes audio in Vorbis format for Ogg containers",
   "Monty <monty@xiph.org>, " "Wim Taymans <wim.taymans@chello.be>",
 };
 
@@ -182,6 +182,8 @@ gst_oggvorbisenc_class_init (OggVorbisEncClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
+  gst_tag_register_musicbrainz_tags ();
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_BITRATE,
       g_param_spec_int ("max-bitrate", "Maximum Bitrate",
           "Specify a maximum bitrate (in bps). Useful for encoding for a fixed-size channel",
