@@ -179,6 +179,8 @@ gst_resample_scale (gst_resample_t * r, void *i_buf, unsigned int i_size)
   r->i_end = r->i_start + r->i_inc * r->i_samples;
 
   r->o_samples = floor (r->i_end - r->halftaps * r->i_inc);
+  if (r->o_samples < 0)
+    r->o_samples = 0;
 
   o_size = r->o_samples * r->channels * 2;
   r->o_buf = r->get_buffer (r->priv, o_size);
@@ -545,6 +547,8 @@ gst_resample_sinc_ft_s16 (gst_resample_t * r)
   o_ptr = (signed short *) r->o_buf;
 
   center = r->o_start;
+  while (center - r->halftaps < -1 * r->filter_length)
+    center += 1.0;
   start_x = center - r->halftaps;
   start_f = floor (start_x);
   start_x -= start_f;
