@@ -25,7 +25,8 @@
 
 #include "xoverlay.h"
 
-enum {
+enum
+{
   HAVE_XWINDOW_ID,
   DESIRED_SIZE,
   LAST_SIGNAL
@@ -54,44 +55,40 @@ gst_x_overlay_get_type (void)
     };
 
     gst_x_overlay_type = g_type_register_static (G_TYPE_INTERFACE,
-						 "GstXOverlay",
-						 &gst_x_overlay_info, 0);
+        "GstXOverlay", &gst_x_overlay_info, 0);
     g_type_interface_add_prerequisite (gst_x_overlay_type,
-				       GST_TYPE_IMPLEMENTS_INTERFACE);
+        GST_TYPE_IMPLEMENTS_INTERFACE);
   }
 
   return gst_x_overlay_type;
 }
 
 /* FIXME: evil hack, we should figure out our marshal handling in this interfaces some day */
-extern void gst_marshal_VOID__INT_INT (GClosure *closure, GValue *return_value, guint n_param_values,
-    const GValue *param_values, gpointer invocation_hint, gpointer marshal_data);
+extern void gst_marshal_VOID__INT_INT (GClosure * closure,
+    GValue * return_value, guint n_param_values, const GValue * param_values,
+    gpointer invocation_hint, gpointer marshal_data);
 
 static void
 gst_x_overlay_base_init (gpointer g_class)
 {
   GstXOverlayClass *overlay_class = (GstXOverlayClass *) g_class;
   static gboolean initialized = FALSE;
-  
-  if (! initialized)
-    {
-      gst_x_overlay_signals[HAVE_XWINDOW_ID] =
+
+  if (!initialized) {
+    gst_x_overlay_signals[HAVE_XWINDOW_ID] =
         g_signal_new ("have-xwindow-id",
-                      GST_TYPE_X_OVERLAY, G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (GstXOverlayClass, have_xwindow_id),
-                      NULL, NULL,
-                      g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1,
-		      G_TYPE_INT);
-      gst_x_overlay_signals[DESIRED_SIZE] =
+        GST_TYPE_X_OVERLAY, G_SIGNAL_RUN_LAST,
+        G_STRUCT_OFFSET (GstXOverlayClass, have_xwindow_id),
+        NULL, NULL, g_cclosure_marshal_VOID__INT, G_TYPE_NONE, 1, G_TYPE_INT);
+    gst_x_overlay_signals[DESIRED_SIZE] =
         g_signal_new ("desired-size-changed",
-                      GST_TYPE_X_OVERLAY, G_SIGNAL_RUN_LAST,
-                      G_STRUCT_OFFSET (GstXOverlayClass, desired_size),
-                      NULL, NULL,
-                      gst_marshal_VOID__INT_INT, G_TYPE_NONE, 2,
-		      G_TYPE_INT, G_TYPE_INT);
-      
-      initialized = TRUE;
-    }
+        GST_TYPE_X_OVERLAY, G_SIGNAL_RUN_LAST,
+        G_STRUCT_OFFSET (GstXOverlayClass, desired_size),
+        NULL, NULL,
+        gst_marshal_VOID__INT_INT, G_TYPE_NONE, 2, G_TYPE_INT, G_TYPE_INT);
+
+    initialized = TRUE;
+  }
 
   overlay_class->set_xwindow_id = NULL;
 }
@@ -107,7 +104,7 @@ gst_x_overlay_base_init (gpointer g_class)
  * stop using that window and create an internal one.
  */
 void
-gst_x_overlay_set_xwindow_id (GstXOverlay *overlay, XID xwindow_id)
+gst_x_overlay_set_xwindow_id (GstXOverlay * overlay, gulong xwindow_id)
 {
   GstXOverlayClass *klass = GST_X_OVERLAY_GET_CLASS (overlay);
 
@@ -126,13 +123,13 @@ gst_x_overlay_set_xwindow_id (GstXOverlay *overlay, XID xwindow_id)
  * This function should be used by video overlay developpers.
  */
 void
-gst_x_overlay_got_xwindow_id (GstXOverlay *overlay, XID xwindow_id)
+gst_x_overlay_got_xwindow_id (GstXOverlay * overlay, gulong xwindow_id)
 {
   g_return_if_fail (overlay != NULL);
   g_return_if_fail (GST_IS_X_OVERLAY (overlay));
-  
+
   g_signal_emit (G_OBJECT (overlay),
-                 gst_x_overlay_signals[HAVE_XWINDOW_ID], 0, (gint) xwindow_id);
+      gst_x_overlay_signals[HAVE_XWINDOW_ID], 0, (gint) xwindow_id);
 }
 
 /**
@@ -145,18 +142,20 @@ gst_x_overlay_got_xwindow_id (GstXOverlay *overlay, XID xwindow_id)
  * size, width and height are set to 0.
  */
 void
-gst_x_overlay_get_desired_size (GstXOverlay *overlay, guint *width, guint *height)
+gst_x_overlay_get_desired_size (GstXOverlay * overlay, guint * width,
+    guint * height)
 {
   guint width_tmp, height_tmp;
   GstXOverlayClass *klass;
-  
+
   g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE ((overlay), GST_TYPE_X_OVERLAY));
-  
+
   klass = GST_X_OVERLAY_GET_CLASS (overlay);
   if (klass->get_desired_size && GST_IS_X_OVERLAY (overlay)) {
     /* this ensures that elements don't need to check width and height for NULL 
        but apps may use NULL */
-    klass->get_desired_size (overlay, width ? width : &width_tmp, height ? height : &height_tmp);
+    klass->get_desired_size (overlay, width ? width : &width_tmp,
+        height ? height : &height_tmp);
   } else {
     if (width)
       *width = 0;
@@ -176,12 +175,13 @@ gst_x_overlay_get_desired_size (GstXOverlay *overlay, guint *width, guint *heigh
  * This function should be used by video overlay developpers.
  */
 void
-gst_x_overlay_got_desired_size (GstXOverlay *overlay, guint width, guint height)
+gst_x_overlay_got_desired_size (GstXOverlay * overlay, guint width,
+    guint height)
 {
   g_return_if_fail (GST_IS_X_OVERLAY (overlay));
-  
+
   g_signal_emit (G_OBJECT (overlay),
-                 gst_x_overlay_signals[DESIRED_SIZE], 0, width, height);
+      gst_x_overlay_signals[DESIRED_SIZE], 0, width, height);
 }
 
 /**
@@ -192,7 +192,7 @@ gst_x_overlay_got_desired_size (GstXOverlay *overlay, guint width, guint height)
  * in the drawable even if the pipeline is PAUSED.
  */
 void
-gst_x_overlay_expose (GstXOverlay *overlay)
+gst_x_overlay_expose (GstXOverlay * overlay)
 {
   GstXOverlayClass *klass = GST_X_OVERLAY_GET_CLASS (overlay);
 
