@@ -151,21 +151,8 @@ gst_adder_link (GstPad * pad, const GstCaps * caps)
 
     if (otherpad != pad) {
       ret = gst_pad_try_set_caps (otherpad, caps);
-      if (GST_PAD_LINK_FAILED (ret)) {
-        return ret;
-      }
-    }
-    pads = g_list_next (pads);
-  }
-
-
-  pads = gst_element_get_pad_list (GST_ELEMENT (adder));
-  while (pads) {
-    GstPad *otherpad = GST_PAD (pads->data);
-
-    if (otherpad != pad) {
-      ret = gst_pad_try_set_caps (otherpad, caps);
-      if (GST_PAD_LINK_FAILED (ret)) {
+      if (ret == GST_PAD_LINK_REFUSED ||
+          (otherpad == adder->srcpad && GST_PAD_LINK_FAILED (ret))) {
         return ret;
       }
     }
@@ -246,7 +233,7 @@ gst_adder_init (GstAdder * adder)
       (&gst_adder_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (adder), adder->srcpad);
   gst_element_set_loop_function (GST_ELEMENT (adder), gst_adder_loop);
-  gst_pad_set_getcaps_function (adder->srcpad, gst_pad_proxy_getcaps);
+  gst_pad_set_getcaps_function (adder->srcpad, gst_adder_get_caps);
   gst_pad_set_link_function (adder->srcpad, gst_adder_link);
 
   adder->format = GST_ADDER_FORMAT_UNSET;
