@@ -140,7 +140,7 @@ gst_audio_convert_fill_compatible (GstAudioConvert * this)
     }
 
     if (pos1_0 != -1 && pos1_1 != -1 && pos2_0 != -1) {
-      this->matrix[pos1_0][pos2_0] = -1.0;
+      this->matrix[pos1_0][pos2_0] = 1.0;
       this->matrix[pos1_1][pos2_0] = 1.0;
     }
 
@@ -161,7 +161,7 @@ gst_audio_convert_fill_compatible (GstAudioConvert * this)
     }
 
     if (pos1_0 != -1 && pos1_1 != -1 && pos2_0 != -1) {
-      this->matrix[pos2_0][pos1_0] = -1.0;
+      this->matrix[pos2_0][pos1_0] = 1.0;
       this->matrix[pos2_0][pos1_1] = 1.0;
     }
   }
@@ -242,7 +242,7 @@ gst_audio_convert_fill_one_other (gfloat ** matrix,
     GstAudioChannelPosition to_pos_r,
     GstAudioChannelPosition to_pos_c, gfloat ratio)
 {
-  gfloat in_r, out_r[2];
+  gfloat in_r, out_r[2] = { 0, };
 
   /*
    * The idea is that we add up from the input (which means that if we
@@ -350,10 +350,10 @@ gst_audio_convert_fill_others (GstAudioConvert * this)
         GST_AUDIO_CHANNEL_POSITION_REAR_CENTER, RATIO_FRONT_REAR);
   } else if (in_has_center && !out_has_center && out_has_front) {
     gst_audio_convert_fill_one_other (this->matrix,
-        &this->sinkcaps, in_r,
-        GST_AUDIO_CHANNEL_POSITION_REAR_LEFT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_RIGHT,
-        GST_AUDIO_CHANNEL_POSITION_REAR_CENTER,
+        &this->sinkcaps, in_c,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT_OF_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT_OF_CENTER,
+        GST_AUDIO_CHANNEL_POSITION_FRONT_CENTER,
         &this->srccaps, out_f,
         GST_AUDIO_CHANNEL_POSITION_FRONT_LEFT,
         GST_AUDIO_CHANNEL_POSITION_FRONT_RIGHT,
