@@ -189,8 +189,7 @@ gst_audioscale_class_init (AudioscaleClass * klass)
 }
 
 static GstStaticCaps gst_audioscale_passthru_caps =
-    GST_STATIC_CAPS
-    ("audio/x-raw-int, channels = [ 3, MAX ]");
+GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 3, MAX ]");
 static GstStaticCaps gst_audioscale_convert_caps =
 GST_STATIC_CAPS ("audio/x-raw-int, channels = [ 1, 2 ]");
 
@@ -396,6 +395,7 @@ gst_audioscale_get_buffer (void *priv, unsigned int size)
   audioscale->outbuf = gst_buffer_new ();
   GST_BUFFER_SIZE (audioscale->outbuf) = size;
   GST_BUFFER_DATA (audioscale->outbuf) = g_malloc (size);
+  GST_BUFFER_OFFSET (audioscale->outbuf) = audioscale->gst_resample_offset;
   GST_BUFFER_TIMESTAMP (audioscale->outbuf) =
       audioscale->gst_resample_offset * GST_SECOND /
       audioscale->gst_resample->o_rate;
@@ -564,34 +564,61 @@ gst_audioscale_chain (GstPad * pad, GstData * _data)
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
 
+  if (audioscale->gst_resample && audioscale->saved_discont) {
+    GstData *saved_event = GST_DATA (audioscale->saved_discont);
+
+    audioscale->saved_discont = NULL;
+    GST_DEBUG ("processing saved discont event");
+    gst_audioscale_chain (pad, saved_event);
+  }
+
   if (GST_IS_EVENT (_data)) {
     GstEvent *e = GST_EVENT (_data);
 
     switch (GST_EVENT_TYPE (e)) {
       case GST_EVENT_DISCONTINUOUS:{
-        gint64 new_off = 0;
+        GstEvent *new_event;
+        gint64 ts = (gint64) GST_CLOCK_TIME_NONE;
+        gint64 off = (gint64) GST_BUFFER_OFFSET_NONE;
 
         if (!audioscale->gst_resample) {
-          GST_LOG ("Discont before negotiation took place - ignoring");
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_TIME, &new_off)) {
+          if (audioscale->saved_discont)
+            gst_data_unref (GST_DATA (audioscale->saved_discont));
+          audioscale->saved_discont = e;
+          GST_DEBUG ("Discont before negotiation happened, will process later");
+          return;               /* process it later when we're set up */
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_TIME, &ts)) {
           /* time -> out-sample */
-          new_off = new_off * audioscale->gst_resample->o_rate / GST_SECOND;
-        } else if (gst_event_discont_get_value (e,
-                GST_FORMAT_DEFAULT, &new_off)) {
+          off = ts * audioscale->gst_resample->o_rate / GST_SECOND;
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_DEFAULT, &off)) {
           /* in-sample -> out-sample */
-          new_off *= audioscale->gst_resample->o_rate;
-          new_off /= audioscale->gst_resample->i_rate;
-        } else if (gst_event_discont_get_value (e, GST_FORMAT_BYTES, &new_off)) {
-          new_off /= audioscale->gst_resample->channels;
-          new_off /=
-              (audioscale->gst_resample->format == GST_RESAMPLE_S16) ? 2 : 4;
-          new_off *= audioscale->gst_resample->o_rate;
-          new_off /= audioscale->gst_resample->i_rate;
+          off *= audioscale->gst_resample->o_rate;
+          off /= audioscale->gst_resample->i_rate;
+          ts = off * GST_SECOND / audioscale->gst_resample->o_rate;
+        } else if (gst_event_discont_get_value (e, GST_FORMAT_BYTES, &off)) {
+          GST_DEBUG ("Discont event, bytes = %" G_GUINT64_FORMAT, off);
+          off /= audioscale->gst_resample->channels;
+          off /= (audioscale->gst_resample->format == GST_RESAMPLE_S16) ? 2 : 4;
+          off *= audioscale->gst_resample->o_rate;
+          off /= audioscale->gst_resample->i_rate;
+          ts = off * GST_SECOND / audioscale->gst_resample->o_rate;
         } else {
           /* *sigh* */
           GST_DEBUG ("Discont without value - ignoring");
+          gst_data_unref (_data);
+          return;
         }
-        audioscale->gst_resample_offset = new_off;
+        audioscale->gst_resample_offset = off;
+        GST_DEBUG ("New resample offset after discont event: %"
+            G_GUINT64_FORMAT ", %" GST_TIME_FORMAT, off, GST_TIME_ARGS (ts));
+        /* need to create a new discont event with a corrected offset, or
+         * downstream might get an offset that refers to our input rate */
+        new_event =
+            gst_event_new_discontinuous (GST_EVENT_DISCONT_NEW_MEDIA (e),
+            GST_FORMAT_DEFAULT, off, GST_FORMAT_TIME, ts, GST_FORMAT_UNDEFINED);
+        gst_data_unref (_data);
+        e = new_event;
+        _data = GST_DATA (new_event);
         /* fall-through */
       }
       default:
