@@ -205,7 +205,7 @@ gst_festival_chain (GstPad * pad, GstData * _data)
   gchar *wavefile;
   int filesize;
   FILE *fd;
-  char *p;
+  guchar *p;
   char ack[4];
   int n;
   GstFestival *festival;
@@ -226,6 +226,7 @@ gst_festival_chain (GstPad * pad, GstData * _data)
   size = GST_BUFFER_SIZE (buf);
 
   /* Copy text over to server, escaping any quotes */
+  fprintf (fd, "(Parameter.set 'Audio_Required_Rate 16000)\n");
   fprintf (fd, "(tts_textall \"\n");
   for (p = GST_BUFFER_DATA (buf); p && (*p != '\0') && size; p++, size--) {
     if ((*p == '"') || (*p == '\\'))
@@ -254,7 +255,7 @@ gst_festival_chain (GstPad * pad, GstData * _data)
 
     if (wavefile) {
       outbuf = gst_buffer_new ();
-      GST_BUFFER_DATA (outbuf) = wavefile;
+      GST_BUFFER_DATA (outbuf) = (guchar *) wavefile;
       GST_BUFFER_SIZE (outbuf) = filesize;
 
       gst_pad_push (festival->srcpad, GST_DATA (outbuf));
