@@ -87,7 +87,9 @@ alawdec_getcaps (GstPad * pad)
     otherpad = alawdec->sinkpad;
     base_caps = gst_caps_new_simple ("audio/x-raw-int",
         "width", G_TYPE_INT, 16, "depth", G_TYPE_INT, 16,
-        "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+        "signed", G_TYPE_BOOLEAN, TRUE,
+        "endianness", G_TYPE_INT, G_BYTE_ORDER,
+        "signed", G_TYPE_BOOLEAN, TRUE, NULL);
   }
   othercaps = gst_pad_get_allowed_caps (otherpad);
 
@@ -116,6 +118,7 @@ alawdec_link (GstPad * pad, const GstCaps * caps)
   GstStructure *structure;
   const GValue *rate, *chans;
   GstCaps *base_caps;
+  GstPadLinkReturn link_return;
 
   structure = gst_caps_get_structure (caps, 0);
   rate = gst_structure_get_value (structure, "rate");
@@ -130,14 +133,19 @@ alawdec_link (GstPad * pad, const GstCaps * caps)
     otherpad = alawdec->srcpad;
     base_caps = gst_caps_new_simple ("audio/x-raw-int",
         "width", G_TYPE_INT, 16, "depth", G_TYPE_INT, 16,
-        "endianness", G_TYPE_INT, G_BYTE_ORDER, NULL);
+        "endianness", G_TYPE_INT, G_BYTE_ORDER,
+        "signed", G_TYPE_BOOLEAN, TRUE, NULL);
   }
 
   structure = gst_caps_get_structure (base_caps, 0);
   gst_structure_set_value (structure, "rate", rate);
   gst_structure_set_value (structure, "channels", chans);
 
-  return gst_pad_try_set_caps (otherpad, base_caps);
+  link_return = gst_pad_try_set_caps (otherpad, base_caps);
+
+  gst_caps_free (base_caps);
+
+  return link_return;
 }
 
 GType
@@ -173,7 +181,7 @@ gst_alawdec_base_init (GstALawDecClass * klass)
     "A Law to PCM conversion",
     "Codec/Decoder/Audio",
     "Convert 8bit A law to 16bit PCM",
-    "Zaheer Merali <zaheer@bellworldwide.net>"
+    "Zaheer Abbas Merali <zaheerabbas at merali dot org>"
   };
 
   gst_element_class_add_pad_template (element_class, alawdec_src_template);
