@@ -27,12 +27,20 @@
 
 static GstFormat scr_format;
 
-
 GST_DEBUG_CATEGORY_STATIC (gstmpegparse_debug);
 #define GST_CAT_DEFAULT (gstmpegparse_debug)
 
 GST_DEBUG_CATEGORY_EXTERN (GST_CAT_SEEK);
 
+#define MP_INVALID_SCR ((guint64)(-1))
+#define MP_MUX_RATE_MULT 50
+#define MP_MIN_VALID_BSS 8192
+#define MP_MAX_VALID_BSS 16384
+/*
+ * Hysteresis value to limit the
+ * total predicted time skipping about
+ */
+#define MP_SCR_RATE_HYST 0.08
 
 /* elementfactory information */
 static GstElementDetails mpeg_parse_details = {
@@ -44,7 +52,7 @@ static GstElementDetails mpeg_parse_details = {
 
 #define CLASS(o)	GST_MPEG_PARSE_CLASS (G_OBJECT_GET_CLASS (o))
 
-#define DEFAULT_MAX_DISCONT	10000
+#define DEFAULT_MAX_DISCONT	120000
 
 /* GstMPEGParse signals and args */
 enum
@@ -86,13 +94,19 @@ static void gst_mpeg_parse_set_clock (GstElement * element, GstClock * clock);
 static gboolean gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse,
     GstBuffer * buffer);
 
+static void gst_mpeg_parse_reset (GstMPEGParse * mpeg_parse);
 static void gst_mpeg_parse_handle_discont (GstMPEGParse * mpeg_parse,
     GstEvent * event);
+static void gst_mpeg_parse_reset (GstMPEGParse * mpeg_parse);
 
 static void gst_mpeg_parse_send_data (GstMPEGParse * mpeg_parse, GstData * data,
     GstClockTime time);
 static void gst_mpeg_parse_send_discont (GstMPEGParse * mpeg_parse,
     GstClockTime time);
+static void gst_mpeg_parse_send_event (GstMPEGParse * mpeg_parse,
+    GstEvent * event, GstClockTime time);
+
+static void gst_mpeg_parse_new_pad (GstElement * element, GstPad * pad);
 
 static void gst_mpeg_parse_loop (GstElement * element);
 
@@ -161,7 +175,7 @@ gst_mpeg_parse_class_init (GstMPEGParseClass * klass)
           FALSE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MAX_DISCONT,
       g_param_spec_int ("max_discont", "Max Discont",
-          "The maximun allowed SCR discontinuity", 0, G_MAXINT,
+          "The maximum allowed SCR discontinuity", 0, G_MAXINT,
           DEFAULT_MAX_DISCONT, G_PARAM_READWRITE));
   /* FIXME: Default is TRUE to make the behavior backwards compatible.
      It probably should be FALSE. */
@@ -172,6 +186,7 @@ gst_mpeg_parse_class_init (GstMPEGParseClass * klass)
   gobject_class->get_property = gst_mpeg_parse_get_property;
   gobject_class->set_property = gst_mpeg_parse_set_property;
 
+  gstelement_class->new_pad = gst_mpeg_parse_new_pad;
   gstelement_class->change_state = gst_mpeg_parse_change_state;
   gstelement_class->set_clock = gst_mpeg_parse_set_clock;
   gstelement_class->get_index = gst_mpeg_parse_get_index;
@@ -185,6 +200,7 @@ gst_mpeg_parse_class_init (GstMPEGParseClass * klass)
   klass->handle_discont = gst_mpeg_parse_handle_discont;
   klass->send_data = gst_mpeg_parse_send_data;
   klass->send_discont = gst_mpeg_parse_send_discont;
+  klass->send_event = gst_mpeg_parse_send_event;
 
   /* FIXME: this is a hack.  We add the pad templates here instead
    * in the base_init function, since the derived class (mpegdemux)
@@ -198,31 +214,34 @@ gst_mpeg_parse_class_init (GstMPEGParseClass * klass)
 static void
 gst_mpeg_parse_init (GstMPEGParse * mpeg_parse)
 {
-  mpeg_parse->sinkpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&sink_factory),
-      "sink");
+  GstElementClass *klass = GST_ELEMENT_GET_CLASS (mpeg_parse);
+  GstPadTemplate *templ;
+
+  templ = gst_element_class_get_pad_template (klass, "sink");
+  mpeg_parse->sinkpad = gst_pad_new_from_template (templ, "sink");
   gst_element_add_pad (GST_ELEMENT (mpeg_parse), mpeg_parse->sinkpad);
   gst_pad_set_formats_function (mpeg_parse->sinkpad,
       gst_mpeg_parse_get_src_formats);
   gst_pad_set_convert_function (mpeg_parse->sinkpad,
       gst_mpeg_parse_convert_src);
 
-  mpeg_parse->srcpad =
-      gst_pad_new_from_template (gst_static_pad_template_get (&src_factory),
-      "src");
-  gst_element_add_pad (GST_ELEMENT (mpeg_parse), mpeg_parse->srcpad);
-  gst_pad_set_formats_function (mpeg_parse->srcpad,
-      gst_mpeg_parse_get_src_formats);
-  gst_pad_set_convert_function (mpeg_parse->srcpad, gst_mpeg_parse_convert_src);
-  gst_pad_set_event_mask_function (mpeg_parse->srcpad,
-      gst_mpeg_parse_get_src_event_masks);
-  gst_pad_set_event_function (mpeg_parse->srcpad,
-      gst_mpeg_parse_handle_src_event);
-  gst_pad_set_query_type_function (mpeg_parse->srcpad,
-      gst_mpeg_parse_get_src_query_types);
-  gst_pad_set_query_function (mpeg_parse->srcpad,
-      gst_mpeg_parse_handle_src_query);
-  gst_pad_use_explicit_caps (mpeg_parse->srcpad);
+  if ((templ = gst_element_class_get_pad_template (klass, "src"))) {
+    mpeg_parse->srcpad = gst_pad_new_from_template (templ, "src");
+    gst_element_add_pad (GST_ELEMENT (mpeg_parse), mpeg_parse->srcpad);
+    gst_pad_set_formats_function (mpeg_parse->srcpad,
+        gst_mpeg_parse_get_src_formats);
+    gst_pad_set_convert_function (mpeg_parse->srcpad,
+        gst_mpeg_parse_convert_src);
+    gst_pad_set_event_mask_function (mpeg_parse->srcpad,
+        gst_mpeg_parse_get_src_event_masks);
+    gst_pad_set_event_function (mpeg_parse->srcpad,
+        gst_mpeg_parse_handle_src_event);
+    gst_pad_set_query_type_function (mpeg_parse->srcpad,
+        gst_mpeg_parse_get_src_query_types);
+    gst_pad_set_query_function (mpeg_parse->srcpad,
+        gst_mpeg_parse_handle_src_query);
+    gst_pad_use_explicit_caps (mpeg_parse->srcpad);
+  }
 
   gst_element_set_loop_function (GST_ELEMENT (mpeg_parse), gst_mpeg_parse_loop);
 
@@ -232,8 +251,11 @@ gst_mpeg_parse_init (GstMPEGParse * mpeg_parse)
   mpeg_parse->max_discont = DEFAULT_MAX_DISCONT;
 
   mpeg_parse->do_adjust = TRUE;
+  mpeg_parse->use_adjust = TRUE;
 
   GST_FLAG_SET (mpeg_parse, GST_ELEMENT_EVENT_AWARE);
+
+  gst_mpeg_parse_reset (mpeg_parse);
 }
 
 static void
@@ -259,7 +281,7 @@ gst_mpeg_parse_update_streaminfo (GstMPEGParse * mpeg_parse)
   gst_props_add_entry (props, (GstPropsEntry *) entry);
 
   entry =
-      gst_props_entry_new ("bitrate", G_TYPE_INT (mpeg_parse->mux_rate * 400));
+      gst_props_entry_new ("bitrate", G_TYPE_INT (mpeg_parse->mux_rate * 8));
   gst_props_add_entry (props, (GstPropsEntry *) entry);
 
   caps = gst_caps_new ("mpeg_streaminfo",
@@ -270,6 +292,28 @@ gst_mpeg_parse_update_streaminfo (GstMPEGParse * mpeg_parse)
 }
 #endif
 
+static void
+gst_mpeg_parse_reset (GstMPEGParse * mpeg_parse)
+{
+  mpeg_parse->current_scr = 0;
+  mpeg_parse->current_ts = 0;
+  mpeg_parse->bytes_since_scr = 0;
+  mpeg_parse->avg_bitrate_time = 0;
+  mpeg_parse->avg_bitrate_bytes = 0;
+  mpeg_parse->first_scr = MP_INVALID_SCR;
+  mpeg_parse->first_scr_pos = 0;
+  mpeg_parse->last_scr = MP_INVALID_SCR;
+  mpeg_parse->last_scr_pos = 0;
+  mpeg_parse->scr_rate = 0;
+
+  mpeg_parse->adjust = 0;
+  mpeg_parse->next_scr = 0;
+  mpeg_parse->mux_rate = 0;
+
+  mpeg_parse->discont_pending = FALSE;
+  mpeg_parse->scr_pending = FALSE;
+}
+
 static void
 gst_mpeg_parse_handle_discont (GstMPEGParse * mpeg_parse, GstEvent * event)
 {
@@ -277,14 +321,20 @@ gst_mpeg_parse_handle_discont (GstMPEGParse * mpeg_parse, GstEvent * event)
 
   g_return_if_fail (GST_EVENT_TYPE (event) == GST_EVENT_DISCONTINUOUS);
 
-  if (gst_event_discont_get_value (event, GST_FORMAT_TIME, &time)) {
-    GST_DEBUG_OBJECT (mpeg_parse,
-        "forwarding discontinuity, time: %0.3fs", (double) time / GST_SECOND);
+  if (GST_EVENT_DISCONT_NEW_MEDIA (event)) {
+    gst_mpeg_parse_reset (mpeg_parse);
+  }
+
+  if (gst_event_discont_get_value (event, GST_FORMAT_TIME, (gint64 *) & time)
+      && (GST_CLOCK_TIME_IS_VALID (time))) {
+    GST_DEBUG_OBJECT (mpeg_parse, "forwarding discontinuity, time: %0.3fs",
+        (double) time / GST_SECOND);
 
     if (CLASS (mpeg_parse)->send_discont)
       CLASS (mpeg_parse)->send_discont (mpeg_parse, time);
   } else {
     /* Use the next SCR to send a discontinuous event. */
+    GST_DEBUG_OBJECT (mpeg_parse, "Using next SCR to send discont");
     mpeg_parse->discont_pending = TRUE;
     mpeg_parse->scr_pending = TRUE;
   }
@@ -337,9 +387,50 @@ gst_mpeg_parse_send_discont (GstMPEGParse * mpeg_parse, GstClockTime time)
 {
   GstEvent *event;
 
-  if (GST_PAD_IS_USABLE (mpeg_parse->srcpad)) {
-    event = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
+  event = gst_event_new_discontinuous (FALSE, GST_FORMAT_TIME, time, NULL);
+
+  if (!event) {
+    GST_ELEMENT_ERROR (GST_ELEMENT (mpeg_parse),
+        RESOURCE, FAILED, (NULL), ("Allocation failed"));
+    return;
+  }
+
+  if (CLASS (mpeg_parse)->send_event)
+    CLASS (mpeg_parse)->send_event (mpeg_parse, event, time);
+}
+
+static void
+gst_mpeg_parse_send_event (GstMPEGParse * mpeg_parse, GstEvent * event,
+    GstClockTime time)
+{
+  if (GST_PAD_IS_USABLE (mpeg_parse->srcpad))
     gst_pad_push (mpeg_parse->srcpad, GST_DATA (event));
+  else
+    gst_event_unref (event);
+}
+
+static void
+gst_mpeg_parse_new_pad (GstElement * element, GstPad * pad)
+{
+  GstMPEGParse *mpeg_parse;
+
+  if (GST_PAD_IS_SINK (pad))
+    return;
+
+  mpeg_parse = GST_MPEG_PARSE (element);
+
+  /* For each new added pad, send a discont so it knows about the current
+   * time. This is required because MPEG allows any sort of order of
+   * packets, including setting base time before defining streams or
+   * even adding streams halfway a stream. */
+  if (!mpeg_parse->scr_pending) {
+    GstEvent *event = gst_event_new_discontinuous (FALSE,
+        GST_FORMAT_TIME,
+        (guint64) MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr +
+            mpeg_parse->adjust),
+        GST_FORMAT_UNDEFINED);
+
+    gst_pad_push (pad, GST_DATA (event));
   }
 }
 
@@ -347,7 +438,7 @@ static gboolean
 gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
 {
   guint8 *buf;
-  guint64 scr;
+  guint64 prev_scr, scr, diff;
   guint32 scr1, scr2;
   guint32 new_rate;
 
@@ -390,15 +481,30 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
     new_rate |= ((gint32) buf[1]) << 7;
     new_rate |= buf[2] >> 1;
   }
+  new_rate *= MP_MUX_RATE_MULT;
 
+  prev_scr = mpeg_parse->current_scr;
   mpeg_parse->current_scr = scr;
+  mpeg_parse->current_ts = MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr +
+      mpeg_parse->adjust);
   mpeg_parse->scr_pending = FALSE;
-  mpeg_parse->bytes_since_scr = 0;
 
-  if (mpeg_parse->next_scr == -1) {
+  if (mpeg_parse->next_scr == MP_INVALID_SCR) {
     mpeg_parse->next_scr = mpeg_parse->current_scr;
   }
 
+  if ((mpeg_parse->first_scr == MP_INVALID_SCR) ||
+      (mpeg_parse->current_scr < mpeg_parse->first_scr)) {
+    mpeg_parse->first_scr = mpeg_parse->current_scr;
+    mpeg_parse->first_scr_pos = gst_bytestream_tell (mpeg_parse->packetize->bs);
+  }
+
+  if ((mpeg_parse->last_scr == MP_INVALID_SCR) ||
+      (mpeg_parse->current_scr > mpeg_parse->last_scr)) {
+    mpeg_parse->last_scr = mpeg_parse->current_scr;
+    mpeg_parse->last_scr_pos = gst_bytestream_tell (mpeg_parse->packetize->bs);
+  }
+
   GST_LOG_OBJECT (mpeg_parse,
       "SCR is %" G_GUINT64_FORMAT
       " (%" G_GUINT64_FORMAT ") next: %"
@@ -413,17 +519,27 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
       MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr) -
       MPEGTIME_TO_GSTTIME (mpeg_parse->next_scr));
 
-  if (ABS ((gint64) mpeg_parse->next_scr - (gint64) (scr)) >
-      mpeg_parse->max_discont) {
-    GST_DEBUG ("discontinuity detected; expected: %" G_GUINT64_FORMAT " got: %"
-        G_GUINT64_FORMAT " real:%" G_GINT64_FORMAT " adjust:%" G_GINT64_FORMAT,
-        mpeg_parse->next_scr, mpeg_parse->current_scr + mpeg_parse->adjust,
-        mpeg_parse->current_scr, mpeg_parse->adjust);
-    if (mpeg_parse->do_adjust) {
-      mpeg_parse->adjust +=
-          (gint64) mpeg_parse->next_scr - (gint64) mpeg_parse->current_scr;
+  /* watch out for integer overflows... */
+  if (mpeg_parse->next_scr > scr) {
+    diff = mpeg_parse->next_scr - scr;
+  } else {
+    diff = scr - mpeg_parse->next_scr;
+  }
 
-      GST_DEBUG ("new adjust: %" G_GINT64_FORMAT, mpeg_parse->adjust);
+  if (diff > mpeg_parse->max_discont) {
+    GST_DEBUG ("discontinuity detected; expected: %" G_GUINT64_FORMAT " got: %"
+        G_GUINT64_FORMAT " adjusted:%" G_GINT64_FORMAT " adjust:%"
+        G_GINT64_FORMAT, mpeg_parse->next_scr, mpeg_parse->current_scr,
+        mpeg_parse->current_scr + mpeg_parse->adjust, mpeg_parse->adjust);
+
+    if (mpeg_parse->do_adjust) {
+      if (mpeg_parse->use_adjust) {
+        mpeg_parse->adjust +=
+            (gint64) mpeg_parse->next_scr - (gint64) mpeg_parse->current_scr;
+        GST_DEBUG ("new adjust: %" G_GINT64_FORMAT, mpeg_parse->adjust);
+      }
+    } else {
+      mpeg_parse->discont_pending = TRUE;
     }
   }
 
@@ -435,13 +551,39 @@ gst_mpeg_parse_parse_packhead (GstMPEGParse * mpeg_parse, GstBuffer * buffer)
         GST_FORMAT_TIME, MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr), 0);
   }
 
-  if (mpeg_parse->mux_rate != new_rate) {
-    mpeg_parse->mux_rate = new_rate;
-
-    //gst_mpeg_parse_update_streaminfo (mpeg_parse);
-    GST_DEBUG ("stream is %1.3fMbs", (mpeg_parse->mux_rate * 400) / 1000000.0);
+  if ((mpeg_parse->current_scr > prev_scr) && (diff < mpeg_parse->max_discont)) {
+    mpeg_parse->avg_bitrate_time +=
+        MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr - prev_scr);
+    mpeg_parse->avg_bitrate_bytes += mpeg_parse->bytes_since_scr;
   }
 
+  if (mpeg_parse->mux_rate != new_rate) {
+    if (GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize)) {
+      mpeg_parse->mux_rate = new_rate;
+    } else if (mpeg_parse->avg_bitrate_bytes > MP_MIN_VALID_BSS) {
+      mpeg_parse->mux_rate =
+          GST_SECOND * mpeg_parse->avg_bitrate_bytes /
+          mpeg_parse->avg_bitrate_time;
+    }
+    //gst_mpeg_parse_update_streaminfo (mpeg_parse);
+    GST_DEBUG ("stream current is %1.3fMbs, calculated over %1.3fkB",
+        (mpeg_parse->mux_rate * 8) / 1048576.0,
+        mpeg_parse->bytes_since_scr / 1024.0);
+  }
+
+  if (mpeg_parse->avg_bitrate_bytes) {
+    GST_DEBUG ("stream avg is %1.3fMbs, calculated over %1.3fkB",
+        (float) (mpeg_parse->avg_bitrate_bytes) * 8 * GST_SECOND
+        / mpeg_parse->avg_bitrate_time / 1048576.0,
+        mpeg_parse->avg_bitrate_bytes / 1024.0);
+  }
+
+  if (mpeg_parse->avg_bitrate_bytes > MP_MAX_VALID_BSS) {
+    mpeg_parse->avg_bitrate_bytes = 0;
+    mpeg_parse->avg_bitrate_time = 0;
+  }
+  mpeg_parse->bytes_since_scr = 0;
+
   return TRUE;
 }
 
@@ -560,12 +702,15 @@ gst_mpeg_parse_loop (GstElement * element)
       gst_element_wait (GST_ELEMENT (mpeg_parse), time);
     }
 
-    if (mpeg_parse->current_scr != -1) {
+    if (mpeg_parse->current_scr != MP_INVALID_SCR) {
       guint64 scr, bss, br;
 
       scr = mpeg_parse->current_scr;
       bss = mpeg_parse->bytes_since_scr;
-      br = mpeg_parse->mux_rate * 50;
+      if (mpeg_parse->scr_rate != 0)
+        br = mpeg_parse->scr_rate;
+      else
+        br = mpeg_parse->mux_rate;
 
       if (br) {
         if (GST_MPEG_PACKETIZE_IS_MPEG2 (mpeg_parse->packetize)) {
@@ -574,6 +719,7 @@ gst_mpeg_parse_loop (GstElement * element)
            *
            * mpeg_parse->next_scr = (scr * br + bss * CLOCK_FREQ) / (CLOCK_FREQ + br);
            */
+
           mpeg_parse->next_scr = scr + (bss * CLOCK_FREQ) / br;
         } else {
           /* we are interpolating the scr here */
@@ -585,8 +731,8 @@ gst_mpeg_parse_loop (GstElement * element)
       }
 
       GST_LOG_OBJECT (mpeg_parse, "size: %" G_GINT64_FORMAT
-          ", total since SCR: %" G_GINT64_FORMAT
-          ", next SCR: %" G_GINT64_FORMAT, size, bss, mpeg_parse->next_scr);
+          ", total since SCR: %" G_GINT64_FORMAT ", br: %" G_GINT64_FORMAT
+          ", next SCR: %" G_GINT64_FORMAT, size, bss, br, mpeg_parse->next_scr);
     }
   }
 }
@@ -603,12 +749,81 @@ gst_mpeg_parse_get_src_formats (GstPad * pad)
   return formats;
 }
 
+/*
+ * Return the bitrate to the nearest byte/sec
+ */
+static gboolean
+gst_mpeg_parse_get_rate (GstMPEGParse * mpeg_parse, gint64 * rate)
+{
+  GstFormat time_format = GST_FORMAT_TIME;
+  GstFormat bytes_format = GST_FORMAT_BYTES;
+  gint64 total_time = 0;
+  gint64 total_bytes = 0;
+
+  /* If upstream knows the total time and the total bytes,
+   * use those to compute an average byterate
+   */
+  if (gst_pad_query (GST_PAD_PEER (mpeg_parse->sinkpad),
+          GST_QUERY_TOTAL, &time_format, &total_time)
+      &&
+      gst_pad_query (GST_PAD_PEER (mpeg_parse->sinkpad),
+          GST_QUERY_TOTAL, &bytes_format, &total_bytes)
+      && total_time != 0) {
+    *rate = GST_SECOND * total_bytes / total_time;
+    return TRUE;
+  }
+  *rate = 0;
+
+  if ((mpeg_parse->first_scr != MP_INVALID_SCR) &&
+      (mpeg_parse->last_scr != MP_INVALID_SCR) &&
+      (mpeg_parse->last_scr_pos - mpeg_parse->first_scr_pos > MP_MIN_VALID_BSS)
+      && (mpeg_parse->last_scr != mpeg_parse->first_scr)) {
+    *rate =
+        GST_SECOND * (mpeg_parse->last_scr_pos -
+        mpeg_parse->first_scr_pos) / MPEGTIME_TO_GSTTIME (mpeg_parse->last_scr -
+        mpeg_parse->first_scr);
+  }
+
+  if (*rate == 0 && mpeg_parse->avg_bitrate_time != 0
+      && mpeg_parse->avg_bitrate_bytes > MP_MIN_VALID_BSS) {
+    *rate =
+        GST_SECOND * mpeg_parse->avg_bitrate_bytes /
+        mpeg_parse->avg_bitrate_time;
+  }
+
+  if (*rate != 0) {
+    /*
+     * check if we need to update scr_rate
+     */
+    if ((mpeg_parse->scr_rate == 0) ||
+        (((double) (ABS (mpeg_parse->scr_rate - *rate)) / mpeg_parse->scr_rate)
+            >= MP_SCR_RATE_HYST)) {
+      mpeg_parse->scr_rate = *rate;
+      return TRUE;
+    }
+  }
+
+  if (mpeg_parse->scr_rate != 0) {
+    *rate = mpeg_parse->scr_rate;
+    return TRUE;
+  }
+
+  if (mpeg_parse->mux_rate != 0) {
+    *rate = mpeg_parse->mux_rate;
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 gboolean
 gst_mpeg_parse_convert_src (GstPad * pad, GstFormat src_format,
     gint64 src_value, GstFormat * dest_format, gint64 * dest_value)
 {
   gboolean res = TRUE;
   GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
+  gint64 rate;
+
 
   switch (src_format) {
     case GST_FORMAT_BYTES:
@@ -616,10 +831,11 @@ gst_mpeg_parse_convert_src (GstPad * pad, GstFormat src_format,
         case GST_FORMAT_DEFAULT:
           *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
-          if (mpeg_parse->mux_rate == 0)
+          if (!gst_mpeg_parse_get_rate (mpeg_parse, &rate))
             res = FALSE;
-          else
-            *dest_value = src_value * GST_SECOND / (mpeg_parse->mux_rate * 50);
+          else {
+            *dest_value = GST_SECOND * src_value / rate;
+          }
           break;
         default:
           res = FALSE;
@@ -630,7 +846,11 @@ gst_mpeg_parse_convert_src (GstPad * pad, GstFormat src_format,
         case GST_FORMAT_DEFAULT:
           *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
-          *dest_value = mpeg_parse->mux_rate * 50 * src_value / GST_SECOND;
+          if (!gst_mpeg_parse_get_rate (mpeg_parse, &rate))
+            res = FALSE;
+          else {
+            *dest_value = src_value * rate / GST_SECOND;
+          }
           break;
         default:
           res = FALSE;
@@ -640,6 +860,7 @@ gst_mpeg_parse_convert_src (GstPad * pad, GstFormat src_format,
       res = FALSE;
       break;
   }
+
   return res;
 }
 
@@ -661,8 +882,8 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQueryType type,
 {
   gboolean res = TRUE;
   GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
-  GstFormat src_format;
-  gint64 src_value;
+  GstFormat src_format = GST_FORMAT_UNDEFINED;
+  gint64 src_value = 0;
 
   switch (type) {
     case GST_QUERY_TOTAL:
@@ -671,6 +892,17 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQueryType type,
         case GST_FORMAT_DEFAULT:
           *format = GST_FORMAT_TIME;
           /* fallthrough */
+        case GST_FORMAT_TIME:
+          /*
+           * Try asking upstream if it knows the time - a DVD might know
+           */
+          src_format = GST_FORMAT_TIME;
+          if (gst_pad_query (GST_PAD_PEER (mpeg_parse->sinkpad),
+                  GST_QUERY_TOTAL, &src_format, &src_value)) {
+            res = TRUE;
+            break;
+          }
+          /* Otherwise fallthrough */
         default:
           src_format = GST_FORMAT_BYTES;
           if (!gst_pad_query (GST_PAD_PEER (mpeg_parse->sinkpad),
@@ -689,7 +921,14 @@ gst_mpeg_parse_handle_src_query (GstPad * pad, GstQueryType type,
           /* fallthrough */
         default:
           src_format = GST_FORMAT_TIME;
-          src_value = MPEGTIME_TO_GSTTIME (mpeg_parse->current_scr);
+          if ((mpeg_parse->current_scr == MP_INVALID_SCR) ||
+              (mpeg_parse->first_scr == MP_INVALID_SCR))
+            res = FALSE;
+          else {
+            gint64 cur =
+                (gint64) (mpeg_parse->current_scr) - mpeg_parse->first_scr;
+            src_value = MPEGTIME_TO_GSTTIME (MAX (0, cur));
+          }
           break;
       }
       break;
@@ -718,7 +957,7 @@ gst_mpeg_parse_get_src_event_masks (GstPad * pad)
 }
 
 static gboolean
-index_seek (GstPad * pad, GstEvent * event, guint64 * offset, gint64 * scr)
+index_seek (GstPad * pad, GstEvent * event, gint64 * offset, gint64 * scr)
 {
   GstIndexEntry *entry;
   GstMPEGParse *mpeg_parse = GST_MPEG_PARSE (gst_pad_get_parent (pad));
@@ -746,7 +985,7 @@ index_seek (GstPad * pad, GstEvent * event, guint64 * offset, gint64 * scr)
 }
 
 static gboolean
-normal_seek (GstPad * pad, GstEvent * event, guint64 * offset, gint64 * scr)
+normal_seek (GstPad * pad, GstEvent * event, gint64 * offset, gint64 * scr)
 {
   gboolean res;
   GstFormat format;
@@ -778,8 +1017,8 @@ gst_mpeg_parse_handle_src_event (GstPad * pad, GstEvent * event)
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      guint64 desired_offset;
-      guint64 expected_scr;
+      gint64 desired_offset;
+      gint64 expected_scr = 0;
 
       /* first to to use the index if we have one */
       if (mpeg_parse->index)
@@ -791,13 +1030,17 @@ gst_mpeg_parse_handle_src_event (GstPad * pad, GstEvent * event)
       if (!res)
         break;
 
-      GST_DEBUG ("sending seek to %" G_GINT64_FORMAT, desired_offset);
+      GST_DEBUG ("sending seek to %" G_GINT64_FORMAT " expected SCR: %"
+          G_GUINT64_FORMAT " (%" G_GUINT64_FORMAT ")", desired_offset,
+          expected_scr, MPEGTIME_TO_GSTTIME (expected_scr));
+
       if (gst_bytestream_seek (mpeg_parse->packetize->bs, desired_offset,
               GST_SEEK_METHOD_SET)) {
         mpeg_parse->discont_pending = TRUE;
         mpeg_parse->scr_pending = TRUE;
         mpeg_parse->next_scr = expected_scr;
-        mpeg_parse->current_scr = -1;
+        mpeg_parse->current_scr = MP_INVALID_SCR;
+        mpeg_parse->current_ts = GST_CLOCK_TIME_NONE;
         mpeg_parse->adjust = 0;
         res = TRUE;
       }
@@ -823,15 +1066,7 @@ gst_mpeg_parse_change_state (GstElement * element)
             GST_MPEG_PACKETIZE_SYSTEM);
       }
       /* initialize parser state */
-      mpeg_parse->current_scr = 0;
-      mpeg_parse->bytes_since_scr = 0;
-      mpeg_parse->adjust = 0;
-      mpeg_parse->next_scr = 0;
-
-      /* zero counters (should be done at RUNNING?) */
-      mpeg_parse->mux_rate = 0;
-      mpeg_parse->discont_pending = FALSE;
-      mpeg_parse->scr_pending = FALSE;
+      gst_mpeg_parse_reset (mpeg_parse);
       break;
     case GST_STATE_PAUSED_TO_READY:
       if (mpeg_parse->packetize) {
