@@ -188,6 +188,7 @@ gst_subparse_event (GstPad * pad, GstEvent * event)
               GST_EVENT_SEEK_METHOD (event) == GST_SEEK_METHOD_SET))
         break;
       self->seek_time = GST_EVENT_SEEK_OFFSET (event);
+      self->flush = GST_EVENT_SEEK_FLAGS (event) & GST_SEEK_FLAG_FLUSH;
       res = TRUE;
       break;
     default:
@@ -284,7 +285,7 @@ get_next_line (GstSubparse * self)
       }
     } while (!buf);
     self->textbuf = g_string_append_len (self->textbuf,
-        GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
+        (const gchar *) GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
     gst_buffer_unref (buf);
     /* search for end-of-line again */
     line_end = strchr (self->textbuf->str, '\n');
@@ -551,7 +552,7 @@ parse_mpsub_init (GstSubparse * self)
  */
 
 static GstSubParseFormat
-gst_subparse_buffer_format_autodetect (GstBuffer * buf)
+gst_subparse_buffer_format_autodetect (const guint8 * data, guint size)
 {
   static gboolean need_init_regexps = TRUE;
   static regex_t mdvd_rx;
@@ -559,9 +560,7 @@ gst_subparse_buffer_format_autodetect (GstBuffer * buf)
 
   /* Copy out chars to guard against short non-null-terminated buffers */
   const gint match_chars = 35;
-  gchar *match_str =
-      g_strndup ((const gchar *) GST_BUFFER_DATA (buf), MIN (match_chars,
-          GST_BUFFER_SIZE (buf)));
+  gchar *match_str = g_strndup ((const gchar *) data, MIN (match_chars, size));
 
   if (!match_str)
     return GST_SUB_PARSE_FORMAT_UNKNOWN;
@@ -621,9 +620,10 @@ gst_subparse_format_autodetect (GstSubparse * self)
       buf = GST_BUFFER (data);
     }
   } while (!buf);
-  self->textbuf = g_string_append_len (self->textbuf, GST_BUFFER_DATA (buf),
+  self->textbuf = g_string_append_len (self->textbuf,
+      (gchar *) GST_BUFFER_DATA (buf), GST_BUFFER_SIZE (buf));
+  format = gst_subparse_buffer_format_autodetect (GST_BUFFER_DATA (buf),
       GST_BUFFER_SIZE (buf));
-  format = gst_subparse_buffer_format_autodetect (buf);
   gst_buffer_unref (buf);
   self->parser_detected = TRUE;
   self->parser.type = format;
@@ -725,7 +725,7 @@ gst_subparse_loop (GstElement * element)
           subtitle, start_time, end_time);
 
       buf = gst_buffer_new ();
-      GST_BUFFER_DATA (buf) = subtitle;
+      GST_BUFFER_DATA (buf) = (guint8 *) subtitle;
       GST_BUFFER_SIZE (buf) = strlen (subtitle);
       GST_BUFFER_TIMESTAMP (buf) = start_time;
       GST_BUFFER_DURATION (buf) = end_time - start_time;
@@ -756,41 +756,53 @@ gst_subparse_change_state (GstElement * element)
   return parent_class->change_state (element);
 }
 
-#if 0
-/* typefinding stuff */
-static GstTypeDefinition subparse_definition = {
-  "subparse/x-text",
-  "text/plain",
-  ".sub",
-  gst_subparse_type_find,
-};
-static GstCaps *
-gst_subparse_type_find (GstBuffer * buf, gpointer private)
+/*
+ * Typefind support.
+ */
+
+static GstStaticCaps sub_caps = GST_STATIC_CAPS ("application/x-subtitle");
+
+#define SUB_CAPS (gst_static_caps_get (&sub_caps))
+
+static void
+gst_subparse_type_find (GstTypeFind * tf, gpointer private)
 {
+  const guint8 *data;
   GstSubParseFormat format;
 
-  format = gst_subparse_buffer_format_autodetect (buf);
+  if (!(data = gst_type_find_peek (tf, 0, 36)))
+    return;
+  format = gst_subparse_buffer_format_autodetect (data, 36);
   switch (format) {
     case GST_SUB_PARSE_FORMAT_MDVDSUB:
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "MicroDVD format detected");
-      return gst_caps_new ("subparse_type_find", "text/plain", NULL);
-    case GST_SUB_PARSE_FORMAT_SUBRIP:
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "SubRip format detected");
-      return gst_caps_new ("subparse_type_find", "text/plain", NULL);
-    case GST_SUB_PARSE_FORMAT_UNKNOWN:
-      GST_DEBUG (GST_CAT_PLUGIN_INFO, "no subtitle format detected");
+      GST_DEBUG ("MicroDVD format detected");
       break;
+    case GST_SUB_PARSE_FORMAT_SUBRIP:
+      GST_DEBUG ("SubRip format detected");
+      break;
+    case GST_SUB_PARSE_FORMAT_MPSUB:
+      GST_DEBUG ("SubRip format detected");
+      break;
+    case GST_SUB_PARSE_FORMAT_UNKNOWN:
+      GST_DEBUG ("no subtitle format detected");
+      return;
   }
-  /* don't know which this is */
-  return NULL;
+
+  /* if we're here, it's ok */
+  gst_type_find_suggest (tf, GST_TYPE_FIND_MAXIMUM, SUB_CAPS);
 }
-#endif
 
 static gboolean
 plugin_init (GstPlugin * plugin)
 {
+  static gchar *sub_exts[] = { "srt", "sub", "mpsub", "mdvd", NULL };
+
   GST_DEBUG_CATEGORY_INIT (subparse_debug, "subparse", 0, ".sub parser");
 
+  if (!gst_type_find_register (plugin, "subparse_typefind", GST_RANK_MARGINAL,
+          gst_subparse_type_find, sub_exts, SUB_CAPS, NULL))
+    return FALSE;
+
   return gst_element_register (plugin, "subparse",
       GST_RANK_PRIMARY, GST_TYPE_SUBPARSE);
 }
