@@ -136,11 +136,11 @@ gst_y4mencode_sinkconnect (GstPad * pad, const GstCaps * caps)
   gdouble fps;
   gdouble framerates[] = {
     00.000,
-    23.976, 24.000,             /* 24fps movie */
+    24000 / 1001., 24.000,      /* 24fps movie */
     25.000,                     /* PAL */
-    29.970, 30.000,             /* NTSC */
+    30000 / 1001., 30.000,      /* NTSC */
     50.000,
-    59.940, 60.000
+    60000 / 1001., 60.000
   };
   GstStructure *structure;
 
@@ -213,9 +213,9 @@ gst_y4mencode_chain (GstPad * pad, GstData * _data)
     header = "FRAME\n";
   }
 
-  snprintf (GST_BUFFER_DATA (outbuf), 255, header,
+  g_snprintf ((gchar *) GST_BUFFER_DATA (outbuf), 255, header,
       filter->width, filter->height, filter->fps_idx);
-  len = strlen (GST_BUFFER_DATA (outbuf));
+  len = strlen ((gchar *) GST_BUFFER_DATA (outbuf));
 
   memcpy (GST_BUFFER_DATA (outbuf) + len, GST_BUFFER_DATA (buf),
       GST_BUFFER_SIZE (buf));
