@@ -119,6 +119,7 @@ struct _GstXImageSink {
 
   gdouble framerate;
   GMutex *x_lock;
+  GMutex *stream_lock;
 
   /* Unused */
   gint pixel_width, pixel_height;
@@ -130,7 +131,11 @@ struct _GstXImageSink {
   GSList *image_pool;
 
   gboolean synchronous;
-  gboolean sw_scaling_failed;
+  gboolean sw_scaling_failed;  
+  gboolean need_expose;
+
+  GMutex *nav_lock;
+  GSList *pend_nav_events;
 };
 
 struct _GstXImageSinkClass {
